from dataclasses import dataclass
from exsited.sdlize.ab_base_dto import ABBaseDTO
from typing import Union


@dataclass(kw_only=True)
class TaxCodeDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: float = None
    link: str = None


@dataclass(kw_only=True)
class AccountingCodeDTO(ABBaseDTO):
    salesRevenue: str = None


@dataclass(kw_only=True)
class PricingLevelDTO(ABBaseDTO):
    name: str = None
    uuid: str = None
    isLinked: str = None
    taxExempt: str = None
    salePriceEnteredIsInclusiveOfTax: str = None
    taxCode: str = None
    salePriceIsBasedOn: str = None


@dataclass(kw_only=True)
class PricingModuleDTO(ABBaseDTO):
    price: str = None
    currency: str = None
    uom: str = None
    lastPurchasePrice: str = None
    averagePurchasePrice: str = None


@dataclass(kw_only=True)
class UomDTO(ABBaseDTO):
    name: str = None
    baseUom: str = None
    saleConversionRate: str = None
    purchaseConversionRate: str = None
    usedForSale: str = None
    usedForPurchase: str = None
    isBase: str = None
    isUsedForSale: str = None
    isUsedForPurchase: str = None


@dataclass(kw_only=True)
class CustomAttributesDTO(ABBaseDTO):
    name: str
    value: Union[str, list] = None


@dataclass(kw_only=True)
class PricingDTO(ABBaseDTO):
    type: str = None
    version: str = None
    latestUsedPricingVersion: str = None
    pricingModule: list[PricingModuleDTO] = None


@dataclass(kw_only=True)
class CodesDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class DimensionDTO(ABBaseDTO):
    uom: str = None
    value: str = None


@dataclass(kw_only=True)
class CurrenciesDTO(ABBaseDTO):
    name: str = None
    usedForSale: str = None
    defaultForSale: str = None
    usedForPurchase: str = None
    defaultForPurchase: str = None
    isUsedForSale: str = None
    isDefaultForSale: str = None
    isUsedForPurchase: str = None
    isDefaultForPurchase: str = None


@dataclass(kw_only=True)
class SaleTaxConfigurationDTO(ABBaseDTO):
    salePriceEnteredIsInclusiveOfTax: str = None
    salePriceIsBasedOn: str = None
    taxCode: TaxCodeDTO = None


@dataclass(kw_only=True)
class SaleChargePropertiesDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class SaleChargeDTO(ABBaseDTO):
    type: str = None
    pricePeriod: str = None
    properties: list[SaleChargePropertiesDTO] = None


@dataclass(kw_only=True)
class TaxConfigurationDTO(ABBaseDTO):
    purchasePriceEnteredIsInclusiveOfTax: str = None
    taxCode: TaxCodeDTO = None


@dataclass(kw_only=True)
class PurchaseSuppliersDTO(ABBaseDTO):
    id: str = None
    name: str = None
    accountingCode: AccountingCodeDTO = None
    purchaseOrderNote: str = None
    defaultPurchaseCurrency: str = None
    defaultPurchasePrice: str = None
    isTaxExemptWhenPurchase: str = None
    taxConfiguration: TaxConfigurationDTO = None
    pricing: PricingDTO = None


@dataclass(kw_only=True)
class SaleDTO(ABBaseDTO):
    isEnable: str = None
    enabled: str = None
    invoiceNote: str = None
    accountingCode: AccountingCodeDTO = None
    defaultSalePrice: str = None
    shippingProfile: str = None
    taxExemptWhenSold: str = None
    pricingMethod: str = None
    taxConfiguration: SaleTaxConfigurationDTO = None
    pricingProfile: dict = None
    pricingSchedules: list[str] = None
    pricingLevels: list[PricingLevelDTO] = None
    charge: SaleChargeDTO = None
    paymentProperties: list[SaleChargePropertiesDTO] = None
    discountProfile: dict = None
    pricing: PricingDTO = None
    width: DimensionDTO = None
    height: DimensionDTO = None
    length: DimensionDTO = None
    weight: DimensionDTO = None
    useOnSalePrice: str= None
    salePriceVariant: str = None
    salePrice: str = None
    startDate: str = None
    endDate: str = None


@dataclass(kw_only=True)
class PurchaseTaxConfigurationDTO(ABBaseDTO):
    purchasePriceEnteredIsInclusiveOfTax: str = None


@dataclass(kw_only=True)
class PurchasePropertiesDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class PurchaseDTO(ABBaseDTO):
    isEnable: str = None
    enabled: str = None
    enableSupplierManagement: str = None
    accountingCode: AccountingCodeDTO = None
    purchaseOrderNote: str = None
    defaultPurchasePrice: str = None
    taxExemptWhenPurchase: str = None
    taxConfiguration: PurchaseTaxConfigurationDTO = None
    pricingProfile: dict = None
    purchaseProperties: list[PurchasePropertiesDTO] = None
    pricing: PricingDTO = None
    suppliers: list[PurchaseSuppliersDTO] = None


@dataclass(kw_only=True)
class InventoryWarehouseDTO(ABBaseDTO):
    name: str = None
    uuid: str = None
    link: str = None
    quantityOnHand: str = None
    quantityOnHandValue: str = None
    quantityPromised: str = None
    quantityOnOrder: str = None
    quantityOnReturn: str = None
    quantityOnPurchaseReturn: str = None
    quantityAvailable: str = None
    quantityAvailableValue: str = None
    uom: str = None

    quantity: str = None
    serialNumber: str = None
    batchNumber: str = None
    expiryDate: str = None
    accountingCode: str = None


@dataclass(kw_only=True)
class InventoryPropertiesDTO(ABBaseDTO):
    quantityAvailableForSaleDetermination: list = None
    quantityAvailableForSale: str = None
    enableLowStockNotification: str = None
    lowStockThresholdIsBasedOn: str = None
    enableReordering: str = None
    reorderingThresholdIsBasedOn: str = None

    lowStockThreshold: str = None
    stockRequiredForSale: str = None
    useLastPurchasePrice: str = None
    preferredSupplier: str = None
    reorderingThreshold: str = None
    enablePreordering: str = None
    enableSerialization: str = None
    enableBatchTracking: str = None
    useAtpAsQuantityAvailableForWebhook: str = None


@dataclass(kw_only=True)
class InventoriesDTO(ABBaseDTO):
    isEnabled: str = None
    preorderPeriod: list[str] = None
    warehouseIsEnabled: str = None
    warehouses: list[InventoryWarehouseDTO] = None
    inventoryProperties: InventoryPropertiesDTO = None

    quantity: str = None
    serialNumber: str = None
    batchNumber: str = None
    expiryDate: str = None
    accountingCode: str = None
    defaultWarehouse: str = None


@dataclass(kw_only=True)
class KpisDTO(ABBaseDTO):
    totalRevenue: float = None
    totalCollected: float = None
    totalOutstanding: float = None
    totalOverdue: float = None


@dataclass(kw_only=True)
class CustomFormDTO(ABBaseDTO):
    name: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ItemDataDTO(ABBaseDTO):
    status: str = None
    id: str = None
    name: str = None
    displayName: str = None
    description: str = None
    type: str = None
    baseUom: str = None
    parentItemId: str = None
    origin: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    imageUri: str = None
    group: dict = None
    manager: str = None
    customForm: CustomFormDTO = None
    customAttributes: list[CustomAttributesDTO] = None
    customObjects: list[str] = None
    codes: list[CodesDTO] = None
    uoms: list[UomDTO] = None
    currencies: list[CurrenciesDTO] = None
    sale: SaleDTO = None
    purchase: PurchaseDTO = None
    inventories: InventoriesDTO = None
    kpis: KpisDTO = None
    invoiceNote: str = None
    pricingLevels: str = None
    accountingCode: str = None
    defaultSalePrice: str = None
    isTaxExemptWhenSold: str = None
    pricing_method: str = None
    pricing_schedules: str = None
    charge: SaleChargeDTO = None
    paymentProperties: list[SaleChargePropertiesDTO] = None
    pricing: PricingDTO = None


@dataclass(kw_only=True)
class PaginationDTO(ABBaseDTO):
    records: int = None
    limit: int = None
    offset: int = None
    previousPage: str = None
    nextPage: str = None


@dataclass(kw_only=True)
class ItemListResponseDTO(ABBaseDTO):
    items: list[ItemDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class ItemResponseDTO(ABBaseDTO):
    item: ItemDataDTO = None


@dataclass(kw_only=True)
class ItemActionDTO(ABBaseDTO):
    items: ItemDataDTO = None

@dataclass(kw_only=True)
class SaleResponseDTO(ABBaseDTO):
    sale: SaleDTO = None


@dataclass(kw_only=True)
class ItemSaleResponseDTO(ABBaseDTO):
    item: SaleResponseDTO = None


@dataclass(kw_only=True)
class PurchaseResponseDTO(ABBaseDTO):
    purchase: PurchaseDTO = None


@dataclass(kw_only=True)
class ItemPurchaseResponseDTO(ABBaseDTO):
    item: PurchaseResponseDTO = None


@dataclass(kw_only=True)
class InventoryResponseDTO(ABBaseDTO):
    inventories: InventoriesDTO = None


@dataclass(kw_only=True)
class ItemInventoryResponseDTO(ABBaseDTO):
    item: InventoryResponseDTO = None



from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import PaginationDTO, CustomAttributesDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class RefundDataDTO(ABBaseDTO):
    status: str = None
    id: str = None
    amount: str = None
    reference: str = None
    note: str = None
    paymentMethod: str = None
    paymentProcessor: str = None
    gatewayResponse: str = None
    creditNoteId: str = None
    customAttributes: list[CustomAttributesDTO] = None
    version: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    date: str = None


@dataclass(kw_only=True)
class RefundDetailsDTO(ABBaseDTO):
    refund: RefundDataDTO = None


@dataclass(kw_only=True)
class RefundListDTO(ABBaseDTO):
    refunds: list[RefundDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class AccountRefundListDTO(ABBaseDTO):
    account: RefundListDTO = None



from exsited.sdlize.dto_base import DTOBase, PropsModType


class ABBaseDTO(DTOBase):
    def set_serialize_conf(self):
        self._props_mod_type = PropsModType.CAMEL_TO_SNAKE

    def set_deserialize_conf(self):
        self._props_mod_type = PropsModType.SNAKE_TO_CAMEL




from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.purchase_order.dto.purchase_order_dto import PurchaseOrderDetailsDTO, PurchaseOrderListDTO, \
    PurchaseOrderDTO, PurchaseOrderCreateDTO, PurchaseOrderDataDTO, PurchaseOrderLineUuidDetailsDTO
from exsited.exsited.purchase_order.purchase_order_api_url import PurchaseOrderApiUrl
from exsited.common.sdk_util import SDKUtil
from exsited.http.ab_rest_processor import ABRestProcessor


class PurchaseOrder(ABRestProcessor):

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> PurchaseOrderListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=PurchaseOrderApiUrl.PURCHASE_ORDERS, params=params, response_obj=PurchaseOrderListDTO())
        return response

    def details(self, id: str) -> PurchaseOrderDetailsDTO:
        response = self.get(url=PurchaseOrderApiUrl.PURCHASE_ORDER_DETAILS.format(id=id),
                            response_obj=PurchaseOrderDetailsDTO())
        return response

    def delete(self, id: str) -> PurchaseOrderDetailsDTO:
        response = self.delete_request(url=PurchaseOrderApiUrl.PURCHASE_ORDER_DELETE.format(id=id),
                            response_obj=PurchaseOrderDetailsDTO())
        return response

    def reactivate(self, id: str) -> PurchaseOrderDetailsDTO:

        response = self.post(url=PurchaseOrderApiUrl.PURCHASE_ORDER_REACTIVATE.format(id=id),
                             response_obj=PurchaseOrderDetailsDTO())
        return response

    def cancel(self, id: str) -> PurchaseOrderDetailsDTO:
        response = self.post(url=PurchaseOrderApiUrl.PURCHASE_ORDER_CANCEL.format(id=id),
                             response_obj=PurchaseOrderDetailsDTO())
        return response

    def information(self, id: str) -> PurchaseOrderDetailsDTO:
        response = self.get(url=PurchaseOrderApiUrl.PURCHASE_ORDER_INFO.format(id=id),
                            response_obj=PurchaseOrderDetailsDTO())
        return response

    def line_uuid(self, id: str, uuid:str) -> PurchaseOrderLineUuidDetailsDTO:
        response = self.get(url=PurchaseOrderApiUrl.PURCHASE_ORDER_LINE_UUID.format(id=id, uuid=uuid),
                            response_obj=PurchaseOrderLineUuidDetailsDTO())
        return response

    def po_line(self, id: str) -> PurchaseOrderDetailsDTO:
        response = self.get(url=PurchaseOrderApiUrl.PURCHASE_ORDER_LINE.format(id=id),
                            response_obj=PurchaseOrderDetailsDTO())
        return response

    def create(self, request_data: PurchaseOrderCreateDTO) -> PurchaseOrderDetailsDTO:
        response = self.post(url=PurchaseOrderApiUrl.PURCHASE_ORDERS,
                             request_obj=request_data,
                             response_obj=PurchaseOrderDetailsDTO())
        return response



from dataclasses import dataclass
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class ReturnItemDTO(ABBaseDTO):
    itemId: str = None
    itemName: str = None
    itemUuid: str = None
    rmaReturned: str = None
    rmaRequested: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ReturnMerchandiseAuthorisationDTO(ABBaseDTO):
    id: str = None
    customerReturnStatus: str = None
    customerReturnDate: str = None
    invoiceId: str = None
    note: str = None
    receivedCount: str = None
    returns: list[ReturnItemDTO] = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None


@dataclass(kw_only=True)
class PaginationDTO(ABBaseDTO):
    records: int = None
    limit: int = None
    offset: int = None
    previousPage: str = None
    nextPage: str = None


@dataclass(kw_only=True)
class ReturnMerchandiseAuthorisationListDTO(ABBaseDTO):
    returnMerchandiseAuthorisations: list[ReturnMerchandiseAuthorisationDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class ReturnMerchandiseAuthorisationDetailsDTO(ABBaseDTO):
    returnMerchandiseAuthorisation: ReturnMerchandiseAuthorisationDTO = None



@dataclass(kw_only=True)
class InvoiceReturnMerchandiseAuthorisationListDTO(ABBaseDTO):
    invoice: ReturnMerchandiseAuthorisationListDTO = None


@dataclass(kw_only=True)
class InvoiceReturnMerchandiseAuthorisationDetailsDTO(ABBaseDTO):
    invoice: ReturnMerchandiseAuthorisationDetailsDTO = None



@dataclass(kw_only=True)
class ReceiveReturnItemDTO(ABBaseDTO):
    rmaReceiveQuantity: str = None
    itemId: str = None
    itemName: str = None
    itemUuid: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ReceiveReturnMerchandiseAuthorisationDTO(ABBaseDTO):
    id: str = None
    receiveReturns: list[ReceiveReturnItemDTO] = None
    createdBy: str = None
    createdOn: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ReceiveReturnMerchandiseAuthorisationListDTO(ABBaseDTO):
    receiveReturnMerchandiseAuthorisations: list[ReceiveReturnMerchandiseAuthorisationDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class ReceiveReturnMerchandiseAuthorisationDetailsDTO(ABBaseDTO):
    receiveReturnMerchandiseAuthorisation: ReceiveReturnMerchandiseAuthorisationDTO = None

@dataclass(kw_only=True)
class RmaReturnDataDTO(ABBaseDTO):
    rmaQuantity: str = None
    itemId: str = None
    itemName: str = None
    itemUuid: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ReturnMerchandiseAuthorisationCreationDTO(ABBaseDTO):
    id: str = None  # System-generated if not provided
    date: str = None
    note: str = None
    returns: list[RmaReturnDataDTO] = None

@dataclass(kw_only=True)
class CreateReturnMerchandiseAuthorisationDTO(ABBaseDTO):
    returnMerchandiseAuthorisations: ReturnMerchandiseAuthorisationCreationDTO = None


@dataclass(kw_only=True)
class ReceiveReturnDTO(ABBaseDTO):
    rmaReceiveQuantity: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ReceiveReturnMerchandiseAuthorisationsListDTO(ABBaseDTO):
    receiveReturns: list[ReceiveReturnDTO] = None


@dataclass(kw_only=True)
class CreateReceiveReturnMerchandiseAuthorisationsDTO(ABBaseDTO):
    receiveReturnMerchandiseAuthorisations: ReceiveReturnMerchandiseAuthorisationsListDTO = None


@dataclass(kw_only=True)
class CreateReceiveReturnMerchandiseAuthorisationDetailsDTO(ABBaseDTO):
    returnMerchandiseAuthorisation: ReturnMerchandiseAuthorisationDTO = None



class AccountApiUrl:
    ACCOUNTS = "/api/v2/accounts"
    ACCOUNTS_V3 = "/api/v3/accounts"
    ACCOUNT_INFORMATION = "/api/v3/accounts/{id}/information"
    ACCOUNT_CANCEL = "/api/v3/accounts/{id}/cancel"
    ACCOUNT_REACTIVATE = "/api/v2/accounts/{id}/reactivate"
    ACCOUNT_REACTIVATE_V3 = "/api/v3/accounts/{id}/reactivate"
    ACCOUNT_UPDATE_INFORMATION = "/api/v3/accounts/{id}/information"
    ACCOUNT_DELETE = "/api/v3/accounts/{id}"
    ACCOUNT_CONTACTS = "/api/v3/accounts/{id}/contacts"
    ACCOUNT_CONTACT_TYPE = "/api/v3/accounts/{id}/contacts/{contact_type}"
    ACCOUNT_CONTACT_UPDATE = "/api/v2/accounts/{id}/contacts/{contact_type}"
    ACCOUNT_CONTACT_UPDATE_V3 = "/api/v3/accounts/{id}/contacts/{contact_type}"
    ACCOUNT_CONTACT_DELETE = "/api/v3/accounts/{id}/contacts/{contact_type}"
    ACCOUNT_PAYMENT_METHODS = "/api/v2/accounts/{id}/payment-methods"
    ACCOUNT_PAYMENT_METHODS_V3 = "/api/v3/accounts/{id}/payment-methods"
    EACH_PAYMENT_METHODS = "/api/v2/accounts/{id}/payment-methods/{reference}"
    EACH_PAYMENT_METHODS_V3 = "/api/v3/accounts/{id}/payment-methods/{reference}"
    ACCOUNT_BILLING_PREFERENCE = "/api/v3/accounts/{id}/billing-preferences"
    ACCOUNT_ADDRESSES = "/api/v3/accounts/{id}/addresses"
    ACCOUNT_PAYMENT_METHODS_UPDATE = "/api/v3/accounts/{id}/payment-methods/{reference}"
    ACCOUNT_CONTACT_MODIFY = "/api/v3/accounts/{id}/contacts/{contact_type}"
    ACCOUNTS_ADDRESS = "/api/v3/accounts/{id}/addresses"
    ACCOUNTS_NOTE = "/api/v3/accounts/{id}/notes"
    ACCOUNTS_ADDRESS_UUID = "/api/v3/accounts/{id}/addresses/{uuid}"
    ACCOUNTS_NOTE_UUID = "/api/v3/accounts/{id}/notes/{uuid}"
    ACCOUNTS_NOTE_UUID_FILES = "/api/v3/accounts/{id}/notes/{uuid}/files"
    ACCOUNTS_NOTE_UUID_FILES_UUID = "/api/v3/accounts/{id}/notes/{uuid}/files/{file_uuid}"
    ACCOUNTS_IMAGE = "/api/v3/accounts/{id}/image"
    ACCOUNT_SENT_EMAIL = "/api/v3/accounts/{id}/communications/emails"
    ACCOUNT_SENT_EMAIL_UUID = "/api/v3/accounts/{id}/communications/emails/{uuid}"



class GiftCertificatesApiUrl:
    GIFT_CERTIFICATES = "/api/v3/gift_certificates"
    GIFT_CERTIFICATE_DETAIL = "/api/v3/gift_certificates/{uuid}"
    GIFT_CERTIFICATE_ALLOCATIONS = "/api/v3/gift_certificates/{uuid}/allocations"
    GIFT_CERTIFICATE_TRANSACTIONS = "/api/v3/gift_certificates/{uuid}/transactions"
    GIFT_CERTIFICATE_CREATE = "/api/v3/gift_certificates"
    GIFT_CERTIFICATE_DISABLE = "/api/v3/gift_certificates/{uuid}/disable"
    GIFT_CERTIFICATE_ENABLE = "/api/v3/gift_certificates/{uuid}/enable"
    GIFT_CERTIFICATE_ALLOCATE = "/api/v3/gift_certificates/{uuid}/allocate"
    GIFT_CERTIFICATE_DEALLOCATE = "/api/v3/gift_certificates/{uuid}/deallocate"
    GIFT_CERTIFICATE_AMEND = "/api/v3/gift_certificates/{uuid}/amend"



from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import TaxDTO, PaginationDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO
from exsited.exsited.common.dto.common_dto import PaginationDTO

@dataclass(kw_only=True)
class PurchaseOrderCurrencyDTO(ABBaseDTO):
    uuid: str = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class PurchaseOrderPricingRuleDTO(ABBaseDTO):
    uuid: str = None
    version: str = None
    priceType: str = None
    price: str = None
    uom: str = None
    warehouse: str = None
    pricingVersion: str = None
    latestUsedPricingVersion: str = None


@dataclass(kw_only=True)
class PurchaseOrderItemPriceSnapshotDTO(ABBaseDTO):
    pricingRule: PurchaseOrderPricingRuleDTO = None


@dataclass(kw_only=True)
class PurchaseOrderTaxCodeDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: str = None
    link: str = None


@dataclass(kw_only=True)
class PurchaseOrderItemPurchaseTaxConfigurationDTO(ABBaseDTO):
    purchasePriceIsTaxInclusive: str = None
    taxCode: PurchaseOrderTaxCodeDTO = None


@dataclass(kw_only=True)
class KPIDTO(ABBaseDTO):
    totalExpense: float = None
    estimatedTotal: float = None
    totalOutstanding: float = None
    totalOverdue: float = None
    lastInvoiceIssueDate: str = None
    lastInvoiceTotal: float = None
    totalPurchaseInvoice: float = None
    lastReactivatedOn: str = None
    lastCalcelledOn: str = None
    lastChangedOn: str = None
    lastDeletedOn: str = None
    issueDate: str = None

@dataclass(kw_only=True)
class PurchaseOrderItemAccountingCodeDTO(ABBaseDTO):
    costOfGoodsSold: str = None


@dataclass(kw_only=True)
class PurchaseOrderLineDTO(ABBaseDTO):
    subtotal: str = None
    total: str = None
    tax: str = None
    itemUuid: str = None
    itemId: str = None
    itemName: str = None
    itemQuantity: str = None
    itemPriceSnapshot: PurchaseOrderItemPriceSnapshotDTO = None
    itemPurchaseTaxConfiguration: PurchaseOrderItemPurchaseTaxConfigurationDTO = None
    itemPriceTaxExempt: str = None
    itemPriceTax: TaxDTO = None
    purchaseOrderNote: str = None
    itemAccountingCode: PurchaseOrderItemAccountingCodeDTO = None
    uuid: str = None
    version: str = None
    itemSerialOrBatchNumber: str = None



@dataclass(kw_only=True)
class PurchaseOrderDTO(ABBaseDTO):
    status: str = None
    id: str = None
    currency: PurchaseOrderCurrencyDTO = None
    supplierInvoiceId: str = None
    issueDate: str = None
    dueDate: str = None
    expectedCompletionDate: str = None
    subtotal: str = None
    tax: str = None
    total: str = None
    priceTaxInclusive: str = None
    purchaseOrderNote: str = None
    accountId: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    customAttributes: list = None  # Added to match response
    customObjects: list = None  # Added to match response
    lines: list[PurchaseOrderLineDTO] = None
    kpis: KPIDTO = None


@dataclass(kw_only=True)
class PurchaseOrderDataDTO(ABBaseDTO):
    status: str = None
    id: str = None
    currency: str = None
    supplierInvoiceId: str = None
    issueDate: str = None
    dueDate: str = None
    expectedCompletionDate: str = None
    subtotal: str = None
    tax: str = None
    total: str = None
    priceTaxInclusive: str = None
    purchaseOrderNote: str = None
    accountId: str = None
    createdBy: str = None
    createdOn: str = None
    uuid: str = None
    version: str = None
    customAttributes: list = None
    customObjects: list = None
    lines: list[PurchaseOrderLineDTO] = None

    effectiveDate: str = None


@dataclass(kw_only=True)
class PurchaseOrderLineUuidDTO(ABBaseDTO):
    line: PurchaseOrderLineDTO = None



@dataclass(kw_only=True)
class PurchaseOrderDetailsDTO(ABBaseDTO):
    purchaseOrder: PurchaseOrderDTO = None


@dataclass(kw_only=True)
class PurchaseOrderLineUuidDetailsDTO(ABBaseDTO):
    purchaseOrder: PurchaseOrderLineUuidDTO = None


@dataclass(kw_only=True)
class PurchaseOrderCreateDTO(ABBaseDTO):
    purchaseOrder: PurchaseOrderDataDTO = None


@dataclass(kw_only=True)
class PurchaseOrderListDTO(ABBaseDTO):
    purchaseOrders: list[PurchaseOrderDTO] = None
    pagination: PaginationDTO = None



from dataclasses import dataclass

from exsited.exsited.common.dto.common_dto import PaginationDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO

@dataclass(kw_only=True)
class CustomAttributesDetailsDTO(ABBaseDTO):
    name: str = None
    value: str = None

@dataclass(kw_only=True)
class CreditNoteDTO(ABBaseDTO):
    status: str = None
    id: str = None
    date: str = None
    amount: str = None
    invoiceId: str = None
    remainingBalance: str = None
    refundable: str = None
    paymentId: str = None
    customAttributes: list[CustomAttributesDetailsDTO] = None
    customObjects: list = None
    version: str = None
    createdBy: str = None
    createdOn: str = None
    updatedBy: str = None
    updatedOn: str = None
    uuid: str = None
    accountId: str = None


@dataclass(kw_only=True)
class CreditNoteApplicationDTO(ABBaseDTO):
    date: str = None
    amount: str = None
    creditNoteId: str = None
    paymentId: str = None
    refundId: str = None
    remainingBalance: str = None
    createdBy: str = None
    createdOn: str = None
    uuid: str = None
    version: str = None



@dataclass(kw_only=True)
class CreditNoteDetailsDTO(ABBaseDTO):
    creditNote: CreditNoteDTO = None


@dataclass(kw_only=True)
class CreditNoteApplicationListDTO(ABBaseDTO):
    creditNoteApplications: list[CreditNoteApplicationDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class CreditNoteApplicationDetailDTO(ABBaseDTO):
    creditNoteApplication: CreditNoteApplicationDTO = None

@dataclass(kw_only=True)
class InvoiceCreditNoteApplicationListDTO(ABBaseDTO):
    invoice: CreditNoteApplicationListDTO = None




from exsited.exsited.express.dto.express_dto import ExpressDTO
from exsited.exsited.express.express_api_url import ExpressApiUrl
from exsited.exsited.order.dto.order_dto import OrderDetailsDTO
from exsited.http.ab_rest_processor import ABRestProcessor


class Express(ABRestProcessor):

    def create(self, request_data: ExpressDTO) -> OrderDetailsDTO:
        response = self.post(url=ExpressApiUrl.EXPRESS_CREATE, request_obj=request_data, response_obj=OrderDetailsDTO())
        return response



from dataclasses import dataclass
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class RequestTokenDTO(ABBaseDTO):
    grantType: str
    clientId: str
    clientSecret: str
    redirectUri: str
    exsitedUrl: str


@dataclass(kw_only=True)
class RefreshTokenDTO(ABBaseDTO):
    grantType: str = "refresh_token"
    refreshToken: str
    clientId: str
    clientSecret: str
    redirectUri: str


@dataclass(kw_only=True)
class TokenResponseDTO(ABBaseDTO):
    accessToken: str = None
    refreshToken: str = None



from dataclasses import dataclass
from exsited.sdlize.ab_base_dto import ABBaseDTO

@dataclass(kw_only=True)
class CurrencyDTO(ABBaseDTO):
    uuid: str = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class TimeZoneDTO(ABBaseDTO):
    uuid: str = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class TaxDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: str = None
    link: str = None

    amount: str = None


@dataclass(kw_only=True)
class PaginationDTO(ABBaseDTO):
    records: int = None
    limit: int = None
    offset: int = None
    previousPage: str = None
    nextPage: str = None


@dataclass(kw_only=True)
class AddressDTO(ABBaseDTO):
    addressLine1: str = None
    addressLine2: str = None
    addressLine3: str = None
    addressLine4: str = None
    addressLine5: str = None
    postCode: str = None
    city: str = None
    state: str = None
    country: str = None
    isDefaultBilling: bool = None
    isDefaultShipping: bool = None
    uuid: str = None
    name: str = None

@dataclass(kw_only=True)
class ShippingProfileDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    invoiceNote: str = None
    type: str = None
    fixedAmount: str = None
    isTaxExempt: str = None
    isTaxInclusive: str = None
    taxConfiguration: TaxDTO = None
    accountingCode: str = None


@dataclass(kw_only=True)
class CustomAttributesDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class CustomObjectDTO(ABBaseDTO):
    name: str = None
    uuid: str = None
    link: str = None


@dataclass(kw_only=True)
class CustomFormsDTO(ABBaseDTO):
    uuid: str = None
    name: str = None


from dataclasses import dataclass
from typing import Union
from exsited.exsited.item.dto.item_dto import PaginationDTO
from exsited.exsited.order.dto.order_dto import CustomAttributesDataDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class AllocationDTO(ABBaseDTO):
    account: str = None
    type: str = None
    date: str = None


@dataclass(kw_only=True)
class TransactionDTO(ABBaseDTO):
    accountingCode: str = None
    type: str = None
    date: str = None
    amount: float = None
    currency: str = None
    reference: str = None


@dataclass(kw_only=True)
class CodeDTO(ABBaseDTO):
    length: str = None
    prefix: str = None


@dataclass(kw_only=True)
class CustomAttributeDataDTO(ABBaseDTO):
    name: str = None
    id: str = None


@dataclass(kw_only=True)
class CustomAttributeResponseDataDTO(ABBaseDTO):
    attribute: list[CustomAttributeDataDTO] = None
    value: str = None


@dataclass(kw_only=True)
class GiftCertificateDTO(ABBaseDTO):
    status: str = None
    account: str = None
    accountingCode: str = None

    amount: str = None
    remainingBalance: str = None
    usedAmount: str = None
    currency: str = None
    expiryDate: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None

    allocations: list[AllocationDTO] = None
    transactions: list[TransactionDTO] = None

    try:
        code: str = None
        customAttributes: list[CustomAttributeResponseDataDTO] = None
    except:
        code: CodeDTO = None
        customAttributes: list[CustomAttributesDataDTO] = None


@dataclass(kw_only=True)
class GiftCertificateResponseDTO(ABBaseDTO):
    status: str = None
    account: str = None
    accountingCode: str = None

    amount: str = None
    remainingBalance: str = None
    usedAmount: str = None
    currency: str = None
    expiryDate: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None

    allocations: list[AllocationDTO] = None
    transactions: list[TransactionDTO] = None


    code: CodeDTO = None
    customAttributes: list[CustomAttributesDataDTO] = None


@dataclass(kw_only=True)
class AllocationListDataDTO(ABBaseDTO):
    allocations: list[AllocationDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class TransactionListDataDTO(ABBaseDTO):
    transactions: list[TransactionDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class GiftCertificatesListDTO(ABBaseDTO):
    giftCertificates: list[GiftCertificateDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class GiftCertificatesListResponseDTO(ABBaseDTO):
    giftCertificates: list[GiftCertificateResponseDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class GiftCertificateAllocationListDTO(ABBaseDTO):
    giftCertificate: AllocationListDataDTO = None


@dataclass(kw_only=True)
class GiftCertificateTransactionsListDTO(ABBaseDTO):
    giftCertificate: TransactionListDataDTO = None


@dataclass(kw_only=True)
class GiftCertificateDetailsDTO(ABBaseDTO):
    giftCertificate: GiftCertificateDTO = None


@dataclass(kw_only=True)
class GiftCertificateDetailsResponseDTO(ABBaseDTO):
    giftCertificate: GiftCertificateResponseDTO = None


class RefundApiUrl:
    DETAILS = "/api/v3/refunds/{id}"
    ACCOUNT_REFUND_LIST = "/api/v3/accounts/{id}/refunds"
    CREATE = '/api/v3/credit-notes/{cn_id}/refunds'
    REFUND_DELETE = '/api/v3/refunds/{id}'



from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.payment.dto.payment_dto import PaymentDetailsDTO, PaymentCreateDTO, CardPaymentCreateDTO, \
    CardDirectDebitPaymentCreateDTO, PaymentInvoiceResponseDTO, PaymentListDTO, PaymentAccountResponseDTO, \
    PaymentOrderResponseDTO, PaymentMultipleRequestDTO
from exsited.exsited.payment.payment_api_url import PaymentApiUrl
from exsited.common.sdk_util import SDKUtil
from exsited.http.ab_rest_processor import ABRestProcessor

class Payment(ABRestProcessor):

    def create(self, invoice_id: str, request_data: PaymentCreateDTO) -> PaymentDetailsDTO:
        response = self.post(url=PaymentApiUrl.PAYMENT_CREATE.format(invoice_id=invoice_id), request_obj=request_data, response_obj=PaymentDetailsDTO())
        return response

    def invoice_details(self, id: str) -> PaymentInvoiceResponseDTO:
        response = self.get(url=PaymentApiUrl.PAYMENT_DETAILS_INVOICE.format(id=id), response_obj=PaymentInvoiceResponseDTO)
        return response

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> PaymentListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=PaymentApiUrl.PAYMENT_LIST, params=params, response_obj=PaymentListDTO())
        return response

    def details(self, id: str) -> PaymentDetailsDTO:
        response = self.get(url=PaymentApiUrl.PAYMENT_DETAILS.format(id=id), response_obj=PaymentDetailsDTO())
        return response

    def account_details(self, id: str) -> PaymentAccountResponseDTO:
        response = self.get(url=PaymentApiUrl.PAYMENT_ACCOUNT_DETAILS.format(id=id), response_obj=PaymentAccountResponseDTO())
        return response

    def order_details(self, id: str) -> PaymentOrderResponseDTO:
        response = self.get(url=PaymentApiUrl.PAYMENT_ORDER_DETAILS.format(id=id), response_obj=PaymentOrderResponseDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=PaymentApiUrl.PAYMENT_DELETE.format(id=id))
        return response


from dataclasses import dataclass
from typing import Union
from exsited.exsited.common.dto.common_dto import PaginationDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class PaymentProcessorDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    default: str = None
    name: str = None
    displayName: str = None
    description: str = None
    provider: str = None
    currency: str = None


@dataclass(kw_only=True)
class BillingPreferencesDTO(ABBaseDTO):
    communicationProfile: str = None
    invoiceMode: str = None
    invoiceTerm: str = None
    billingPeriod: str = None
    billingStartDate: str = None
    billingStartDayOfMonth: str = None
    paymentProcessor: str = None
    paymentMode: str = None
    paymentTerm: str = None
    paymentTermAlignment: str = None


@dataclass(kw_only=True)
class ItemGroupDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    imageUri: str = None
    manager: str = None
    billingPreferences: BillingPreferencesDTO = None


@dataclass(kw_only=True)
class PricingLevelsDataDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None


@dataclass(kw_only=True)
class CurrencyDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    country: str = None
    symbol: str = None
    currencyCode: str = None
    precision: str = None
    rounding: str = None
    cashRoundingInterval: str = None


@dataclass(kw_only=True)
class TaxDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    country: str = None
    configuration: str = None
    code: str = None
    rate: str = None


@dataclass(kw_only=True)
class RedemptionCodeDTO(ABBaseDTO):
    codeId: str = None
    note: str = None
    maxLimit: str = None
    usageCount: str = None


@dataclass(kw_only=True)
class DiscountProfileDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    invoiceNote: str = None
    accountingCode: str = None
    discountStartTime: str = None
    discountEndTime: str = None
    discountType: str = None
    fixed: str = None
    percentage: str = None
    maximumUse: str = None
    totalUsageCount: str = None
    maximumUsePerAccount: str = None
    maximumUsePerOrder: str = None
    requiresRedemptionCode: str = None
    redemptionCode: list[RedemptionCodeDTO] = None



@dataclass(kw_only=True)
class WarehouseDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    default: str = None
    name: str = None
    displayName: str = None
    description: str = None
    country: str = None
    state: str = None
    city: str = None
    postCode: str = None


@dataclass(kw_only=True)
class VariationOptionDTO(ABBaseDTO):
    name: str = None
    uuid: str = None


@dataclass(kw_only=True)
class VariationDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    options: list[VariationOptionDTO] = None


@dataclass(kw_only=True)
class OptionDTO(ABBaseDTO):
    name: str = None
    order: int = None


@dataclass(kw_only=True)
class VariationCreateDataDTO(ABBaseDTO):
    name: str = None
    displayName: str = None
    description: str = None
    options: list[Union[str, OptionDTO]] = None


@dataclass(kw_only=True)
class UseInDTO(ABBaseDTO):
    resource: str = None

@dataclass(kw_only=True)
class ComponentDataDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    useIn: list[UseInDTO] = None
    relation: str = None


@dataclass(kw_only=True)
class TaxConfigurationDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: float = None
    link: str = None


@dataclass(kw_only=True)
class ShippingProfileDTO(ABBaseDTO):
    uuid: str = None
    status: str = None
    name: str = None
    displayName: str = None
    description: str = None
    invoiceNote: str = None
    type: str = None
    fixedAmount: str = None
    isTaxExempt: str = None
    isTaxInclusive: str = None
    taxConfiguration: TaxConfigurationDTO = None
    accountingCode: str = None


@dataclass(kw_only=True)
class ShippingProfilesResponseDTO(ABBaseDTO):
    shippingProfiles: list[ShippingProfileDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class ComponentResponseDTO(ABBaseDTO):
    components: list[ComponentDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class VariationCreateRequestDTO(ABBaseDTO):
    variations: VariationCreateDataDTO = None


@dataclass(kw_only=True)
class VariationsResponseDTO(ABBaseDTO):
    variations: list[VariationDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class VariationUpdateResponseDTO(ABBaseDTO):
    variation: VariationDTO = None



@dataclass(kw_only=True)
class WarehousesResponseDTO(ABBaseDTO):
    warehouses: list[WarehouseDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class DiscountProfilesResponseDTO(ABBaseDTO):
    discountProfiles: list[DiscountProfileDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class TaxesResponseDTO(ABBaseDTO):
    taxes: list[TaxDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class CurrenciesResponseDTO(ABBaseDTO):
    currencies: list[CurrencyDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class SettingPaymentProcessorListDTO(ABBaseDTO):
    paymentProcessors: list[PaymentProcessorDTO] = None


@dataclass(kw_only=True)
class SettingPricingLevelsListDTO(ABBaseDTO):
    pricingLevels: list[PricingLevelsDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class ItemGroupListDTO(ABBaseDTO):
    itemGroup: list[ItemGroupDTO] = None



class PaymentApiUrl:
    PAYMENT_CREATE = "/api/v3/invoices/{invoice_id}/payments"
    PAYMENT_CREATE_CARD = "/api/v3/invoices/{invoice_id}/payments"
    PAYMENT_CREATE_CARD_MULTIPLE = "/api/v3/invoices/payments"
    PAYMENT_CREATE_DIRECT_DEBIT = "/api/v3/invoices/{invoice_id}/payments"
    PAYMENT_DETAILS_INVOICE = "/api/v2/invoices/{id}/payments"
    PAYMENT_DETAILS = "/api/v3/payments/{id}"
    PAYMENT_ACCOUNT_DETAILS = "/api/v3/accounts/{id}/payments"
    PAYMENT_ORDER_DETAILS = "/api/v3/orders/{id}/payments"
    PAYMENT_LIST = "/api/v3/payments"
    PAYMENT_DELETE = "/api/v2/payments/{id}"



from exsited.common.sdk_util import SDKUtil
from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.setting.dto.setting_dto import SettingPaymentProcessorListDTO, SettingPricingLevelsListDTO, \
    ItemGroupListDTO, CurrenciesResponseDTO, TaxesResponseDTO, DiscountProfilesResponseDTO, WarehousesResponseDTO, \
    VariationsResponseDTO, ComponentResponseDTO, ShippingProfilesResponseDTO, VariationCreateRequestDTO, \
    VariationUpdateResponseDTO
from exsited.http.ab_rest_processor import ABRestProcessor
from exsited.exsited.setting.setting_api_url import SettingApiUrl


class Setting(ABRestProcessor):
    def get_settings(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None):
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS, params=params, response_obj={})
        return response

    def get_settings_payment_processor(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> SettingPaymentProcessorListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_PAYMENT_PROCESSOR, params=params, response_obj=SettingPaymentProcessorListDTO())
        return response

    def get_settings_pricing_levels(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> SettingPricingLevelsListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_PRICING_LEVELS, params=params, response_obj=SettingPricingLevelsListDTO())
        return response

    def get_settings_item_group(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> ItemGroupListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_ITEM_GROUPS, params=params, response_obj=ItemGroupListDTO())
        return response

    def get_settings_currencies(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> CurrenciesResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_CURRENCIES, params=params, response_obj=CurrenciesResponseDTO())
        return response

    def get_settings_taxes(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> TaxesResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_TAXES, params=params, response_obj=TaxesResponseDTO())
        return response

    def get_settings_discount_profiles(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> DiscountProfilesResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_DISCOUNT_PROFILES, params=params, response_obj=DiscountProfilesResponseDTO())
        return response

    def get_settings_warehouses(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> WarehousesResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_WAREHOUSES, params=params, response_obj=WarehousesResponseDTO())
        return response

    def get_settings_shipping_profiles(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> ShippingProfilesResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_SHIPPING_PROFILES, params=params, response_obj=ShippingProfilesResponseDTO())
        return response

    def get_settings_components(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> ComponentResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_COMPONENT, params=params, response_obj=ComponentResponseDTO())
        return response

    def get_settings_variations(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> VariationsResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=SettingApiUrl.SETTINGS_VARIATIONS, params=params, response_obj=VariationsResponseDTO())
        return response

    def variation_update(self, uuid: str, request_data: VariationCreateRequestDTO) -> VariationUpdateResponseDTO:
        response = self.patch(url=SettingApiUrl.SETTINGS_VARIATIONS_UPDATE.format(uuid=uuid), request_obj=request_data,
                              response_obj=VariationUpdateResponseDTO())
        return response








from exsited.common.sdk_util import SDKUtil
from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.gift_certificates.dto.gift_certificates_dto import GiftCertificatesListDTO, \
    GiftCertificateDetailsDTO, GiftCertificateAllocationListDTO, GiftCertificateTransactionsListDTO, \
    GiftCertificatesListResponseDTO, GiftCertificateDetailsResponseDTO
from exsited.exsited.gift_certificates.gift_certificates_api_url import GiftCertificatesApiUrl
from exsited.http.ab_rest_processor import ABRestProcessor


class GiftCertificates(ABRestProcessor):

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> GiftCertificatesListResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=GiftCertificatesApiUrl.GIFT_CERTIFICATES, params=params,
                            response_obj=GiftCertificatesListResponseDTO())
        return response

    def details(self, id: str) -> GiftCertificateDetailsResponseDTO:
        response = self.get(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_DETAIL.format(uuid=id),
                            response_obj=GiftCertificateDetailsResponseDTO())
        return response

    def allocation_list(self, id: str, limit: int = None, offset: int = None, direction: SortDirection = None,
                        order_by: str = None) -> GiftCertificateAllocationListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_ALLOCATIONS.format(uuid=id), params=params,
                            response_obj=GiftCertificateAllocationListDTO())
        return response

    def transaction_list(self, id: str, limit: int = None, offset: int = None, direction: SortDirection = None,
                         order_by: str = None) -> GiftCertificateTransactionsListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_TRANSACTIONS.format(uuid=id), params=params,
                            response_obj=GiftCertificateTransactionsListDTO())
        return response

    def disable(self, id: str) -> GiftCertificateDetailsDTO:
        response = self.post(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_DISABLE.format(uuid=id),
                             response_obj=GiftCertificateDetailsDTO())
        return response

    def enable(self, id: str) -> GiftCertificateDetailsDTO:
        response = self.post(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_ENABLE.format(uuid=id),
                             response_obj=GiftCertificateDetailsDTO())
        return response

    def allocate(self, id: str, request_data: GiftCertificateDetailsDTO) -> GiftCertificateAllocationListDTO:
        response = self.post(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_ALLOCATE.format(uuid=id),
                             request_obj=request_data, response_obj=GiftCertificateAllocationListDTO())
        return response

    def deallocate(self, id: str, request_data: GiftCertificateDetailsDTO) -> GiftCertificateAllocationListDTO:
        response = self.post(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_DEALLOCATE.format(uuid=id),
                             request_obj=request_data, response_obj=GiftCertificateAllocationListDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=GiftCertificatesApiUrl.GIFT_CERTIFICATE_DETAIL.format(uuid=id),
                                       response_obj=dict())
        return response



from peewee import Model, CharField, DateTimeField, IntegerField, SQL


class BaseModel(Model):
    class Meta:
        database = None


class Order(BaseModel):
    id = IntegerField(primary_key=True)
    account_id = CharField()
    order_id = CharField()
    item_id = CharField()
    item_name = CharField()
    charge_item_uuid = CharField()
    created = DateTimeField(constraints=[SQL('DEFAULT CURRENT_TIMESTAMP')])
    updated = DateTimeField(constraints=[SQL('DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP')])

    class Meta:
        table_name = 'usage_associations'



from exsited.exsited.common.common_enum import SortDirection


class SDKUtil:

    @staticmethod
    def get_dict_value(data: dict, key: str, default=None):
        if not data or not key:
            return default
        elif key in data:
            return data[key]
        return default

    @staticmethod
    def init_dict_if_value(data: dict, key: str, value):
        if value:
            data[key] = value
        return data

    @staticmethod
    def init_pagination_params(params: dict = None, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None):
        if not params:
            params = {}
        params = SDKUtil.init_dict_if_value(params, "limit", limit)
        params = SDKUtil.init_dict_if_value(params, "offset", offset)
        params = SDKUtil.init_dict_if_value(params, "direction", str(direction))
        params = SDKUtil.init_dict_if_value(params, "order_by", order_by)
        return params



class PurchaseOrderApiUrl:
    PURCHASE_ORDERS = "/api/v3/purchase-orders"
    PURCHASE_ORDER_DETAILS = "/api/v3/purchase-orders/{id}"
    PURCHASE_ORDER_DELETE = "/api/v2/purchase-orders/{id}"
    PURCHASE_ORDER_REACTIVATE = '/api/v3/purchase-orders/{id}/reactivate'
    PURCHASE_ORDER_INFO = "/api/v3/purchase-orders/{id}/information"
    PURCHASE_ORDER_LINE_UUID = "/api/v3/purchase-orders/{id}/lines/{uuid}"
    PURCHASE_ORDER_LINE = "/api/v3/purchase-orders/{id}/lines"
    PURCHASE_ORDER_CREATE = "/api/v3/purchase-orders"
    PURCHASE_ORDER_CANCEL = "/api/v3/purchase-orders/{id}/cancel"



class CreditNoteApiUrl:
    DETAILS = '/api/v3/credit-notes/{id}/'
    APPLICATION_LIST = '/api/v3/credit-note-applications'
    APPLICATION_DETAILS = '/api/v3/credit-note-applications/{uuid}'
    INVOICE_CN_APPLICATION_LIST = '/api/v3/invoices/{id}/credit-note-applications'



from exsited.exsited.order.dto.order_dto import OrderDataDTO
from order_usage_db.connect_with_db import DatabaseConnection
from order_usage_db.order_manager import OrderManager
from order_usage_db.order_service import OrderService


class SaveToDB:
    def process_order_data(_order_id: str, _account_id: str, _item_id: str, _item_name: str, _charge_item_uuid: str):
        order_manager = OrderManager('usage_item_association', 'root', '', '127.0.0.1')
        order_manager.connect_to_db()
        order_manager.process_order(
            account_id=_account_id,
            order_id=_order_id,
            item_id=_item_id,
            item_name=_item_name,
            charge_item_uuid=_charge_item_uuid
        )
        order_manager.disconnect_from_db()



class OrderApiUrl:
    ORDERS = "/api/v2/orders"
    ORDER_CANCEL = "/api/v2/orders/{id}/cancel"
    USAGE_ADD = "/api/v3/usages"
    CONTRACT_ORDER_CREATE = "/api/v2/orders"
    ORDER_REACTIVATE = "/api/v2/orders/{id}/reactivate"
    ORDER_PREORDER = "/api/v2/orders/preorder"
    ORDER_CHANGE = "/api/v2/orders/{id}/change"
    ORDER_DELETE = "/api/v3/orders/{id}"
    ORDER_QUOTE_PDF = "api/v2/orders/{id}/pdf"
    ORDER_INFORMATION = "/api/v3/orders/{id}/information"
    ORDER_BILLING_PREFERENCES = "/api/v3/orders/{id}/billing-preferences"
    ORDER_LINES = "/api/v3/orders/{id}/lines"
    ORDER_LINES_CHARGE = "/api/v3/orders/{id}/lines/{uuid}"
    ORDERS_ACCOUNT = "/api/v3/accounts/{id}/orders"
    ORDER_UPGRADE = "/api/v2/orders/{id}/upgrade"
    CONTRACT_ADJUSTMENT = "/api/v2/orders/{id}/contract-adjustment/preview"
    ORDER_UPGRADE_PREVIEW = "/api/v2/orders/{id}/upgrade/preview"
    ORDER_DOWNGRADE = "/api/v2/orders/{id}/downgrade"
    ORDER_DOWNGRADE_PREVIEW = "/api/v2/orders/{id}/downgrade/preview"
    ORDER_LINES_INFORMATION = "/api/v3/orders/{id}/lines/{uuid}/information"
    ORDER_RELINQUISH = "/api/v3/orders/{id}/relinquish"
    CONTRACT_ORDER_CREATE ="/api/v2/orders"
    USAGE_LIST = "/api/v3/usages"
    USAGE_DETAILS = "/api/v3/usages"
    USAGE_MODIFY = "/api/v3/usages/{uuid}"
    USAGE_UPDATE = "/api/v2/usages/{uuid}"
    USAGE_DELETE = "/api/v3/usages/{uuid}"
    PAYMENT_METHODS = "/api/v3/orders/{id}/payment-methods"


    # Purchase Order URLS
    PURCHASE_ORDER_CREATE = "/api/v3/orders"



from exsited.exsited.common.common_enum import SortDirection
from exsited.common.sdk_util import SDKUtil
from exsited.exsited.return_merchandise_authorisations.dto.return_merchandise_authorisations_dto import \
    ReturnMerchandiseAuthorisationListDTO, ReturnMerchandiseAuthorisationDetailsDTO, \
    InvoiceReturnMerchandiseAuthorisationListDTO, InvoiceReturnMerchandiseAuthorisationDetailsDTO, \
    ReceiveReturnMerchandiseAuthorisationListDTO, ReceiveReturnMerchandiseAuthorisationDetailsDTO, \
    CreateReturnMerchandiseAuthorisationDTO, CreateReceiveReturnMerchandiseAuthorisationsDTO, \
    CreateReceiveReturnMerchandiseAuthorisationDetailsDTO
from exsited.exsited.return_merchandise_authorisations.return_merchandise_authorisations_api_url import \
    ReturnMerchandiseAuthorisationsApiUrl
from exsited.http.ab_rest_processor import ABRestProcessor


class ReturnMerchandiseAuthorisations(ABRestProcessor):
    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> ReturnMerchandiseAuthorisationListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.LIST, params=params, response_obj=ReturnMerchandiseAuthorisationListDTO())
        return response

    def receive_list(self, id: str, rma_id: str, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> ReceiveReturnMerchandiseAuthorisationListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.RECEIVE_LIST.format(id=id, rma_id=rma_id), params=params, response_obj=ReceiveReturnMerchandiseAuthorisationListDTO())
        return response

    def details(self, id: str) -> ReturnMerchandiseAuthorisationDetailsDTO:
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.LIST + f"/{id}", response_obj=ReturnMerchandiseAuthorisationDetailsDTO())
        return response

    def receive_details(self, id: str, rma_id: str, rec_id: str) -> ReceiveReturnMerchandiseAuthorisationDetailsDTO:
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.RECEIVE_DETAILS.format(id=id, rma_id=rma_id, rec_id=rec_id), response_obj=ReceiveReturnMerchandiseAuthorisationDetailsDTO())
        return response

    def invoice_list(self, id: str, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> InvoiceReturnMerchandiseAuthorisationListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.INVOICE_RMA_LIST.format(id=id), params=params, response_obj=InvoiceReturnMerchandiseAuthorisationListDTO())
        return response

    def invoice_rma_details(self, id: str, rma_id: str) -> InvoiceReturnMerchandiseAuthorisationDetailsDTO:
        response = self.get(url=ReturnMerchandiseAuthorisationsApiUrl.INVOICE_RMA_DETAILS.format(id=id, rma_id=rma_id), response_obj=InvoiceReturnMerchandiseAuthorisationDetailsDTO())
        return response

    def invoice_rma_create(self, id: str, request_data: CreateReturnMerchandiseAuthorisationDTO) -> InvoiceReturnMerchandiseAuthorisationDetailsDTO:
        response = self.post(url=ReturnMerchandiseAuthorisationsApiUrl.INVOICE_RMA_CREATE.format(id=id), request_obj=request_data, response_obj=InvoiceReturnMerchandiseAuthorisationDetailsDTO())
        return response

    def invoice_receive_rma_create(self, id: str, rma_id: str, request_data: CreateReceiveReturnMerchandiseAuthorisationsDTO) -> CreateReceiveReturnMerchandiseAuthorisationDetailsDTO:
        response = self.post(url=ReturnMerchandiseAuthorisationsApiUrl.INVOICE_RECEIVE_RMA_CREATE.format(id=id, rma_id=rma_id), request_obj=request_data, response_obj=CreateReceiveReturnMerchandiseAuthorisationDetailsDTO())
        return response



from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.invoice.dto.invoice_dto import InvoiceDetailsDTO, InvoiceListDTO, InvoiceCreateDTO, InvoiceDataDTO, \
    InvoiceAccountDTO, InvoiceOrderDetailsDTO, InvoiceOrderListDTO
from exsited.exsited.invoice.invoice_api_url import InvoiceApiUrl
from exsited.common.sdk_util import SDKUtil
from exsited.http.ab_rest_processor import ABRestProcessor


class Invoice(ABRestProcessor):

    def create(self, id: str, request_data: InvoiceCreateDTO) -> InvoiceDetailsDTO:
        response = self.post(url=InvoiceApiUrl.INVOICE_CREATE.format(id=id), request_obj=request_data,
                             response_obj=InvoiceDetailsDTO())
        return response

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> InvoiceListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=InvoiceApiUrl.INVOICES, params=params, response_obj=InvoiceListDTO())
        return response

    def details(self, id: str):
        response = self.get(url=InvoiceApiUrl.INVOICES + f"/{id}", response_obj=InvoiceDetailsDTO())
        return response

    def information(self, id: str):
        response = self.get(url=InvoiceApiUrl.INVOICE_INFORMATION.format(id=id), response_obj=InvoiceDetailsDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=InvoiceApiUrl.INVOICE_DELETE.format(id=id))
        return response

    def invoice_account(self, accountId: str):
        response = self.get(url=InvoiceApiUrl.INVOICE_ACCOUNT.format(id=accountId), response_obj=InvoiceAccountDTO())
        return response

    def match_structure(self, invoice_data):
        invoice_dict = {
         'status': invoice_data.get('status'),
         'id': invoice_data.get('id'),
         'type': invoice_data.get('type'),
         'customerPurchaseOrderId': invoice_data.get('customer_purchase_order_id'),
         'billingStartDate': invoice_data.get('billing_start_date'),
         'alternateBillingStartDate': invoice_data.get('alternate_billing_start_date'),
         'billingEndDate': invoice_data.get('billing_end_date'),
         'alternateBillingEndDate': invoice_data.get('alternate_billing_end_date'),
         'issueDate': invoice_data.get('issue_date'),
         'alternateIssueDate': invoice_data.get('alternate_issue_date'),
         'dueDate': invoice_data.get('due_date'),
         'alternateDueDate': invoice_data.get('alternate_due_date'),
         'subtotal': invoice_data.get('subtotal'),
         'tax': invoice_data.get('tax'),
         'taxTotal': invoice_data.get('tax_total'),
         'total': invoice_data.get('total'),
         'paid': invoice_data.get('paid'),
         'due': invoice_data.get('due'),
         'paymentStatus': invoice_data.get('payment_status'),
         'discountAmount': invoice_data.get('discount_amount'),
         'priceTaxInclusive': invoice_data.get('price_tax_inclusive'),
         'invoiceNote': invoice_data.get('invoice_note'),
         'accountId': invoice_data.get('account_id'),
         'orderId': invoice_data.get('order_id'),
         'createdBy': invoice_data.get('created_by'),
         'createdOn': invoice_data.get('created_on'),
         'lastUpdatedBy': invoice_data.get('last_updated_by'),
         'lastUpdatedOn': invoice_data.get('last_updated_on'),
         'uuid': invoice_data.get('uuid'),
         'version': invoice_data.get('version'),
         'lines': invoice_data.get('lines'),
         'customForms': invoice_data.get('custom_form'),
         'currency': invoice_data.get('currency')
        }

        return invoice_dict

    def invoice_details_against_order(self, order_id: str):
        raw_response = self.get(url=InvoiceApiUrl.EACH_INVOICE.format(id=order_id))

        if raw_response and 'order' in raw_response and 'invoices' in raw_response['order']:
            invoices = raw_response['order']['invoices']
            if invoices:
                invoice_datas = invoices[0]

                invoice_dict = self.match_structure(invoice_datas)
                invoice_details = InvoiceDetailsDTO(invoice=InvoiceDataDTO(**invoice_dict))

            else:
                invoice_details = InvoiceDetailsDTO(invoice=None)
        else:
            invoice_details = InvoiceDetailsDTO(invoice=None)

        return invoice_details

    def invoice_details_list_against_order(self, order_id: str):
        raw_response = self.get(url=InvoiceApiUrl.EACH_INVOICE.format(id=order_id))

        if raw_response and 'order' in raw_response and 'invoices' in raw_response['order']:
            invoices = raw_response['order']['invoices']
            if invoices:
                invoice_details_list = list()
                for invoice in range(0, len(invoices)):
                    invoice_datas = invoices[invoice]

                    invoice_dict = self.match_structure(invoice_datas)
                    invoice_details = InvoiceDetailsDTO(invoice=InvoiceDataDTO(**invoice_dict))
                    invoice_details_list.append(invoice_details)

            else:
                invoice_details_list = InvoiceDetailsDTO(invoice=None)
        else:
            invoice_details_list = InvoiceDetailsDTO(invoice=None)

        return invoice_details_list

    def invoice_details_against_order_v3(self, order_id: str, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) ->InvoiceOrderDetailsDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=InvoiceApiUrl.EACH_INVOICE_V3.format(id=order_id), params=params, response_obj=InvoiceOrderDetailsDTO())
        return response.order.invoices[-1]

    def invoice_list_against_order_v3(self, order_id: str, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) ->InvoiceOrderListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=InvoiceApiUrl.EACH_INVOICE_V3.format(id=order_id), params=params, response_obj=InvoiceOrderListDTO())
        return response


class PurchasePaymentsApiUrl:
    PURCHASE_PAYMENTS = "/api/v3/purchase-payments"
    PURCHASE_PAYMENTS_DETAILS = "/api/v3/purchase-payments/{id}"
    PURCHASE_PAYMENTS_FROM_PURCHASE_INVOICE = "/api/v3/purchase-invoices/{purchase_invoice_id}/purchase-payments"



from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import CustomFormsDTO, CurrencyDTO, PaginationDTO, CustomAttributesDTO, \
    CustomObjectDTO
from exsited.exsited.order.dto.order_nested_dto import OrderLineDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class LastPaymentDTO(ABBaseDTO):
    id: str = None
    amount: str = None
    createdAt: str = None


@dataclass(kw_only=True)
class TaxDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: float = None

@dataclass(kw_only=True)
class LineItemValueDTO(ABBaseDTO):
    tax: TaxDTO = None
    accountingCode: str = None
    lineItemName: str = None
    lineItemInvoiceNote: str = None
    lineItemQuantity: str = None
    lineItemPrice: str = None
    lineItemDiscountAmount: str = None


@dataclass(kw_only=True)
class LineItemOperationDTO(ABBaseDTO):
    operation: str = None
    uuid: str = None
    value: LineItemValueDTO = None


@dataclass(kw_only=True)
class LineOperationDTO(ABBaseDTO):
    operation: str = None
    uuid: str = None
    value: OrderLineDTO = None
    itemOrderQuantity: str = None
    itemPrice: str = None
    itemDiscountAmount: str = None


@dataclass(kw_only=True)
class KpisDTO(ABBaseDTO):
    outstanding: str = None
    overdue: str = None
    lastPaymentDate: str = None
    paymentApplied: str = None
    creditApplied: str = None
    creditIssued: str = None
    lastReactivatedOn: str = None
    lastCancelledOn: str = None
    lastAmendedOn: str = None
    voidedOn: str = None
    deletedOn: str = None


@dataclass(kw_only=True)
class InvoiceDataDTO(ABBaseDTO):
    status: str = None
    id: str = None
    type: str = None
    customerPurchaseOrderId: str = None
    billingStartDate: str = None
    alternateBillingStartDate: str = None
    billingEndDate: str = None
    alternateBillingEndDate: str = None
    issueDate: str = None
    alternateIssueDate: str = None
    dueDate: str = None
    alternateDueDate: str = None
    subtotal: str = None
    shippingCost: str = None
    accountName: str = None
    tax: str = None
    taxTotal: str = None
    total: str = None
    paid: str = None
    due: str = None
    paymentStatus: str = None
    discountAmount: str = None
    priceTaxInclusive: str = None
    invoiceNote: str = None
    accountId: str = None
    orderId: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None

    lines: list[OrderLineDTO] = None

    customForms: CustomFormsDTO = None
    currency: CurrencyDTO = None

    lastPayment: LastPaymentDTO = None
    kpis: KpisDTO = None

    customAttributes: list[CustomAttributesDTO] = None
    customObjects: list[CustomObjectDTO] = None
    customForm: CustomFormsDTO = None


@dataclass(kw_only=True)
class InvoiceDetailsDTO(ABBaseDTO):
    invoice: InvoiceDataDTO = None


@dataclass(kw_only=True)
class InvoiceCreateDTO(ABBaseDTO):
    invoice: InvoiceDataDTO


@dataclass(kw_only=True)
class InvoiceListDTO(ABBaseDTO):
    invoices: list[InvoiceDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class InvoiceAccountDTO(ABBaseDTO):
    account: InvoiceListDTO = None

@dataclass(kw_only=True)
class InvoiceOrderDetailsDTO(ABBaseDTO):
    order: InvoiceListDTO = None


@dataclass(kw_only=True)
class InvoiceOrderListDTO(ABBaseDTO):
    order: InvoiceListDTO = None






from order_usage_db.order_model import Order


class OrderService:
    def __init__(self, db_connection):
        self.db = db_connection.get_db()
        Order._meta.database = self.db

    def create_association_data(self, account_id, order_id, item_id, item_name, charge_item_uuid):
        with self.db.atomic():
            new_order = Order.create(
                account_id=account_id,
                order_id=order_id,
                item_id=item_id,
                item_name=item_name,
                charge_item_uuid=charge_item_uuid
            )
        return new_order

    def get_order_by_id(self, order_id):
        try:
            order = Order.get(Order.order_id == order_id)
            return order
        except Order.DoesNotExist:
            return None



class ABException(Exception):
    raw_response: any = None
    errors: list = []

    def __init__(self, message=None, exception_type: str = None):
        super().__init__(message)
        self.exception_type = exception_type
        self.message = message

    def add_raw_response(self, raw_response: any):
        self.raw_response = raw_response
        return self

    def add_error(self, error: any):
        self.errors.append(error)

    def get_errors(self):
        return self.errors



from peewee import MySQLDatabase


class DatabaseConnection:
    def __init__(self, db_name, user, password, host):
        self.db = MySQLDatabase(db_name, user=user, password=password, host=host)

    def connect(self):
        self.db.connect()

    def close(self):
        if not self.db.is_closed():
            self.db.close()

    def get_db(self):
        return self.db



from exsited.exsited.account.account import Account
from exsited.exsited.auth.dto.token_dto import RequestTokenDTO
from exsited.exsited.credit_note.credit_note import CreditNote
from exsited.exsited.gift_certificates.gift_certificates import GiftCertificates
from exsited.exsited.invoice.invoice import Invoice
from exsited.exsited.order.order import Order
from exsited.exsited.purchase_order.purchase_order import PurchaseOrder
from exsited.exsited.refund.refund import Refund
from exsited.exsited.return_merchandise_authorisations.return_merchandise_authorisations import \
    ReturnMerchandiseAuthorisations
from exsited.exsited.setting.setting import Setting
from exsited.exsited.payment.payment import Payment
from exsited.exsited.express.express import Express
from exsited.exsited.purchase_payments.purchase_payments import PurchasePayments


class ExsitedSDK:
    _request_token_dto: RequestTokenDTO = None
    account: Account = None
    order: Order = None
    invoice: Invoice = None
    setting: Setting = None
    payment: Payment = None
    refund: Refund = None
    purchase_order: PurchaseOrder = None
    gift_certificates: GiftCertificates = None
    credit_note: CreditNote = None
    return_merchandise_authorisations: ReturnMerchandiseAuthorisations = None
    purchase_payments: PurchasePayments = None
    express: Express = None

    def __init__(self, exsited_url: str = None, grant_type: str = None, client_id: str = None,
                 client_secret: str = None, redirect_uri: str = None):
        if grant_type and client_id and client_secret and redirect_uri:
            self._request_token_dto = RequestTokenDTO(
                grantType=grant_type,
                clientId=client_id,
                clientSecret=client_secret,
                redirectUri=redirect_uri,
                exsitedUrl=exsited_url,
            )
            self._init_endpoints()

    def _init_endpoints(self):
        self.account = Account(request_token_dto=self._request_token_dto)
        self.order = Order(request_token_dto=self._request_token_dto)
        self.invoice = Invoice(request_token_dto=self._request_token_dto)
        self.setting = Setting(request_token_dto=self._request_token_dto)
        self.payment = Payment(request_token_dto=self._request_token_dto)
        self.refund = Refund(request_token_dto=self._request_token_dto)
        self.purchase_order = PurchaseOrder(request_token_dto=self._request_token_dto)
        self.gift_certificates = GiftCertificates(request_token_dto=self._request_token_dto)
        self.credit_note = CreditNote(request_token_dto=self._request_token_dto)
        self.return_merchandise_authorisations = ReturnMerchandiseAuthorisations(request_token_dto=self._request_token_dto)
        self.express = Express(request_token_dto=self._request_token_dto)
        self.purchase_payments = PurchasePayments(request_token_dto=self._request_token_dto)


    def init_sdk(self, request_token_dto: RequestTokenDTO) -> 'ExsitedSDK':
        self._request_token_dto = request_token_dto
        self._init_endpoints()
        return self



from exsited.exsited.common.common_enum import SortDirection
from exsited.exsited.order.dto.order_dto import OrderCreateDTO, OrderDetailsDTO, OrderListDTO, OrderCancelResponseDTO, \
    OrderUpgradeDowngradeDTO, OrderDowngradeDetailsDTO, ContractAdjustmentPreviewResponseDTO, OrderDowngradePreviewDTO, \
    ContractAdjustmentPreviewRequestDTO, OrderUpgradePreviewDTO, OrderDowngradeDTO, OrderUpgradeDTO, \
    OrderPaymentMethodsResponseDTO
from exsited.exsited.order.dto.order_nested_dto import ContractAdjustmentPreviewDTO
from exsited.exsited.order.dto.order_dto import OrderCreateDTO, OrderDetailsDTO, OrderListDTO, OrderCancelResponseDTO, \
    AccountOrdersResponseDTO
from exsited.exsited.order.dto.order_nested_dto import OrderLineDTO
from exsited.exsited.order.dto.usage_dto import UsageCreateDTO, UsageDataDTO, UsageListDTO, UsageModifyDataDTO, \
    UsageUpdateDataDTO, MultipleUsageCreateDTO, MultipleUsageResponseDTO
from exsited.exsited.order.order_api_url import OrderApiUrl
from exsited.common.sdk_util import SDKUtil
from exsited.http.ab_rest_processor import ABRestProcessor


class Order(ABRestProcessor):

    def create(self, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDERS, request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> OrderListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=OrderApiUrl.ORDERS, params=params, response_obj=OrderListDTO())
        return response

    def usage_list(self, limit: int = None, offset: int = None, direction: SortDirection = None, order_by: str = None) -> UsageListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=OrderApiUrl.USAGE_LIST, params=params, response_obj=UsageListDTO())
        return response

    def details(self, id: str) -> OrderDetailsDTO:
        response = self.get(url=OrderApiUrl.ORDERS + f"/{id}", response_obj=OrderDetailsDTO())
        return response

    def usage_details(self, uuid: str) -> UsageDataDTO:
        response = self.get(url=OrderApiUrl.USAGE_DETAILS + f"/{uuid}", response_obj=UsageDataDTO())
        return response

    def usage_modify(self, uuid: str, request_data: UsageCreateDTO) -> UsageDataDTO:
        response = self.patch(url=OrderApiUrl.USAGE_MODIFY.format(uuid=uuid),request_obj=request_data, response_obj=UsageModifyDataDTO())
        return response

    def usage_update(self, uuid: str, request_data: UsageCreateDTO) -> UsageDataDTO:
        response = self.put(url=OrderApiUrl.USAGE_UPDATE.format(uuid=uuid),request_obj=request_data, response_obj=UsageUpdateDataDTO())
        return response

    def usage_delete(self, uuid: str):
        response = self.delete_request(url=OrderApiUrl.USAGE_DELETE.format(uuid=uuid))
        return response


    def cancel(self, id: str, effective_date: str) -> OrderCancelResponseDTO:
        response = self.post(url=OrderApiUrl.ORDER_CANCEL.format(id=id),
                             json_dict={"order": {"effective_date": effective_date}},
                             response_obj=OrderCancelResponseDTO())
        return response

    def add_usage(self, request_data: UsageCreateDTO) -> UsageDataDTO:
        response = self.post(url=OrderApiUrl.USAGE_ADD, request_obj=request_data, response_obj=UsageCreateDTO())
        return response

    def add_multiple_usage(self, request_data: MultipleUsageCreateDTO) -> MultipleUsageResponseDTO:
        response = self.post(url=OrderApiUrl.USAGE_ADD, request_obj=request_data, response_obj=MultipleUsageResponseDTO())
        return response

    def create_with_purchase(self, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.PURCHASE_ORDER_CREATE, request_obj=request_data,
                             response_obj=OrderDetailsDTO())
        return response

    def create_with_contract(self, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.CONTRACT_ORDER_CREATE, request_obj=request_data,
                             response_obj=OrderDetailsDTO())
        return response

    def upgrade(self, order_id: str, request_data: OrderUpgradeDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDER_UPGRADE.format(id=order_id), request_obj=request_data,
                             response_obj=OrderDetailsDTO())
        return response

    def contract_adjustment(self, order_id: str, request_data: ContractAdjustmentPreviewRequestDTO) -> ContractAdjustmentPreviewResponseDTO:
        response = self.post(url=OrderApiUrl.CONTRACT_ADJUSTMENT.format(id=order_id), request_obj=request_data,
                             response_obj=ContractAdjustmentPreviewResponseDTO())
        return response

    def upgrade_preview(self, order_id: str, request_data: OrderUpgradePreviewDTO) -> OrderUpgradeDowngradeDTO:
        response = self.post(url=OrderApiUrl.ORDER_UPGRADE_PREVIEW.format(id=order_id), request_obj=request_data,
                             response_obj=OrderUpgradeDowngradeDTO())
        return response

    def downgrade(self, order_id: str, request_data: OrderDowngradeDTO) -> OrderDowngradeDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDER_DOWNGRADE.format(id=order_id), request_obj=request_data,
                             response_obj=OrderDowngradeDetailsDTO())
        return response

    def downgrade_preview(self, order_id: str, request_data: OrderDowngradePreviewDTO) -> OrderUpgradeDowngradeDTO:
        response = self.post(url=OrderApiUrl.ORDER_DOWNGRADE_PREVIEW.format(id=order_id), request_obj=request_data,
                             response_obj=OrderUpgradeDowngradeDTO())
        return response

    def reactivate(self, id: str, effective_date: str) -> OrderCancelResponseDTO:
        response = self.post(url=OrderApiUrl.ORDER_REACTIVATE.format(id=id),
                             json_dict={"order": {"effective_date": effective_date}},
                             response_obj=OrderCancelResponseDTO())
        return response

    def preorder(self, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDER_PREORDER, request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def change(self, id: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDER_CHANGE.format(id=id),
                             request_obj=request_data,
                             response_obj=OrderDetailsDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=OrderApiUrl.ORDER_DELETE.format(id=id), response_obj={})
        return response

    def order_quote_pdf(self, id: str):
        response = self.get(url=OrderApiUrl.ORDER_QUOTE_PDF.format(id=id))
        return response

    def information(self, id: str) -> OrderDetailsDTO:
        response = self.get(url=OrderApiUrl.ORDER_INFORMATION.format(id=id), response_obj=OrderDetailsDTO())
        return response

    def billing_preferences(self, id: str) -> OrderDetailsDTO:
        response = self.get(url=OrderApiUrl.ORDER_BILLING_PREFERENCES.format(id=id), response_obj=OrderDetailsDTO())
        return response

    def lines(self, id: str) -> OrderDetailsDTO:
        response = self.get(url=OrderApiUrl.ORDER_LINES.format(id=id), response_obj=OrderDetailsDTO())
        return response

    def lines_charge(self, id: str, charge_uuid: str) -> OrderDetailsDTO:
        response = self.get(url=OrderApiUrl.ORDER_LINES_CHARGE.format(id=id, uuid=charge_uuid),
                            response_obj=OrderDetailsDTO())
        return response

    def account_list(self, id: str, limit: int = None, offset: int = None, direction: SortDirection = None,
                     order_by: str = None) -> AccountOrdersResponseDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=OrderApiUrl.ORDERS_ACCOUNT.format(id=id), params=params,
                            response_obj=AccountOrdersResponseDTO())
        return response

    def update_information(self, id: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.patch(url=OrderApiUrl.ORDER_INFORMATION.format(id=id), request_obj=request_data,
                              response_obj=OrderDetailsDTO())
        return response

    def update_line_information(self, id: str, uuid: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.patch(url=OrderApiUrl.ORDER_LINES_INFORMATION.format(id=id, uuid=uuid),
                              request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def update_billing_preference(self, id: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.put(url=OrderApiUrl.ORDER_BILLING_PREFERENCES.format(id=id),
                            request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def fill_in_billing_preference(self, id: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.patch(url=OrderApiUrl.ORDER_BILLING_PREFERENCES.format(id=id),
                            request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def relinquish(self, id: str, request_data: OrderCreateDTO) -> OrderDetailsDTO:
        response = self.post(url=OrderApiUrl.ORDER_RELINQUISH.format(id=id),
                             request_obj=request_data, response_obj=OrderDetailsDTO())
        return response

    def payment_methods(self, id: str) -> OrderPaymentMethodsResponseDTO:
        response = self.get(url=OrderApiUrl.PAYMENT_METHODS.format(id=id), response_obj=OrderPaymentMethodsResponseDTO())
        return response


from dataclasses import dataclass
from exsited.exsited.auth.auth_api_url import AuthApiUrl
from exsited.exsited.auth.dto.token_dto import TokenResponseDTO, RequestTokenDTO, RefreshTokenDTO
from exsited.common.ab_exception import ABException
from exsited.common.sdk_conf import SDKConfig
from exsited.common.sdk_console import SDKConsole
from exsited.common.sdk_const import SDKConst
from exsited.common.sdk_util import SDKUtil
from exsited.http.http_const import RequestType
from exsited.http.http_requester import HTTPRequester, HTTPResponse
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass
class HTTPRequestData:
    url: str
    request_type: RequestType
    json_dict: dict = None
    data: dict = None
    params: dict = None
    file: dict = None
    exception: bool = True


class ABRestProcessor:
    http_requester: HTTPRequester = HTTPRequester()
    rest_token: TokenResponseDTO = None
    request_token_dto: RequestTokenDTO = None

    def __init__(self, request_token_dto: RequestTokenDTO):
        self.request_token_dto = request_token_dto

    def process_error_response(self, response: HTTPResponse, response_data: dict):
        exception_message = "Something happened wrong!"
        http_code = response.httpCode
        if http_code == 400:
            exception_message = "Validation Errors"
        if http_code == 403:
            exception_message = "Access denied"

        exception = ABException(exception_message).add_raw_response(response_data)
        errors = SDKUtil.get_dict_value(response_data, "errors")
        if isinstance(errors, list):
            for error in errors:
                message = SDKUtil.get_dict_value(error, "message")
                if message:
                    exception.add_error(message)
        raise exception

    def _get_data(self, response: HTTPResponse, response_obj: ABBaseDTO, exception=True):
        response_data = response.data
        SDKConsole.log(response_data, is_print=SDKConfig.PRINT_RAW_RESPONSE)
        if response.httpCode == 204:
            return {
                "success": True,
                "status_code": 204
            }
        if response.status != SDKConst.SUCCESS or not response_data:
            if exception:
                self.process_error_response(response=response, response_data=response_data)
            return None
        if response_obj:
            return response_obj.load_dict(response_data)
        return response_data

    def _set_token(self, api_response):
        response: TokenResponseDTO = self._get_data(api_response, response_obj=TokenResponseDTO())
        if not response.accessToken or not response.refreshToken:
            raise ABException("Unable to set token")
        self.rest_token = response

    def _init_auth(self):
        api_response = self.http_requester.post(url=AuthApiUrl.GET_TOKEN, json_dict=self.request_token_dto.to_dict())
        self._set_token(api_response=api_response)

    def _renew_token(self):
        refresh_token = RefreshTokenDTO(
            clientId=self.request_token_dto.clientId,
            clientSecret=self.request_token_dto.clientSecret,
            redirectUri=self.request_token_dto.redirectUri,
            refreshToken=self.rest_token.refreshToken
        )
        api_response = self.http_requester.post(url=AuthApiUrl.GET_TOKEN, json_dict=refresh_token.to_dict())
        self._set_token(api_response=api_response)

    def _init_config(self):
        self.http_requester.baseUrl = self.request_token_dto.exsitedUrl
        if not self.rest_token or not self.rest_token.accessToken:
            self._init_auth()
        self.http_requester.add_bearer_token(self.rest_token.accessToken)

    def _send_request(self, request_data: HTTPRequestData) -> HTTPResponse:
        response = None
        if request_data.request_type == RequestType.POST:
            response = self.http_requester.post(url=request_data.url, json_dict=request_data.json_dict, data=request_data.data, file=request_data.file)
        elif request_data.request_type == RequestType.PUT:
            response = self.http_requester.put(url=request_data.url, json_dict=request_data.json_dict, data=request_data.data, file=request_data.file)
        elif request_data.request_type == RequestType.PATCH:
            response = self.http_requester.patch(url=request_data.url, json_dict=request_data.json_dict, data=request_data.data, file=request_data.file)
        elif request_data.request_type == RequestType.DELETE:
            response = self.http_requester.delete(url=request_data.url, params=request_data.params)
        else:
            response = self.http_requester.get(url=request_data.url, params=request_data.params)
        request_summary = f"URL: {self.http_requester.baseUrl} \nURL Postfix: {request_data.url} \nparams: {request_data.params} \nJSON Data: {request_data.json_dict}"
        SDKConsole.log(message=request_summary, is_print=SDKConfig.PRINT_REQUEST_DATA)
        return response

    def process_rest_request(self, request_data: HTTPRequestData, response_obj: ABBaseDTO = None):
        self._init_config()
        response: HTTPResponse = self._send_request(request_data=request_data)
        if response.httpCode == 401:
            self._renew_token()
            response: HTTPResponse = self._send_request(request_data=request_data)
        return self._get_data(response=response, response_obj=response_obj, exception=request_data.exception)

    def get(self, url: str, params: dict = None, response_obj: ABBaseDTO = None, exception: bool = True):
        return self.process_rest_request(request_data=HTTPRequestData(url=url, params=params, request_type=RequestType.GET, exception=exception), response_obj=response_obj)

    def delete_request(self, url: str, params: dict = None, response_obj: ABBaseDTO = None, exception: bool = True):
        return self.process_rest_request(request_data=HTTPRequestData(url=url, params=params, request_type=RequestType.DELETE, exception=exception), response_obj=response_obj)

    def post(self, url: str, request_obj: ABBaseDTO = None, json_dict: dict = None, data: dict = None, file: dict = None, response_obj: ABBaseDTO = None, exception: bool = True):
        if request_obj and not json_dict:
            json_dict = request_obj.to_dict()
        return self.process_rest_request(request_data=HTTPRequestData(url=url, json_dict=json_dict, data=data, file=file, request_type=RequestType.POST, exception=exception), response_obj=response_obj)

    def put(self, url: str, request_obj: ABBaseDTO = None, json_dict: dict = None, data: dict = None, file: dict = None, response_obj: ABBaseDTO = None, exception: bool = True):
        if request_obj and not json_dict:
            json_dict = request_obj.to_dict()
        return self.process_rest_request(request_data=HTTPRequestData(url=url, json_dict=json_dict, data=data, file=file, request_type=RequestType.PUT, exception=exception), response_obj=response_obj)

    def patch(self, url: str, request_obj: ABBaseDTO = None, json_dict: dict = None, data: dict = None, file: dict = None, response_obj: ABBaseDTO = None, exception: bool = True):
        if request_obj and not json_dict:
            json_dict = request_obj.to_dict()
        return self.process_rest_request(request_data=HTTPRequestData(url=url, json_dict=json_dict, data=data, file=file, request_type=RequestType.PATCH, exception=exception), response_obj=response_obj)





from dataclasses import dataclass, field, asdict
from exsited.exsited.common.dto.common_dto import CurrencyDTO, TimeZoneDTO, PaginationDTO, CustomAttributesDTO, TaxDTO, \
    AddressDTO, CustomFormsDTO, CustomObjectDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO
from exsited.exsited.account.dto.account_nested_dto import AccountingCodeDTO, CommunicationPreferenceDTO, \
    PaymentMethodsDataDTO, BillingPreferencesDTO, PaymentMethodsDTO, PaymentCardMethodsDTO, PaymentCardMethodsDataDTO, \
    PaymentMethodListDTO, AccountContacts, AccountContactsType, AccountContactsUpdate, AccountContactUpdate, \
    AcccountAddressDTO, NoteDataDTO, NoteFileDataDTO, NoteFileUuidDataDTO, AccountImageDataDTO, \
    AccountBillingPreferencesDetailsDTO, \
    AccountBillingPreferencesResponseDetailsDTO, AccountAddressRequestDetailsDTO, AccountAddressResponseDetailsDTO, \
    PaymentMethodListDTO, AccountContacts, AccountContactsType, AccountContactsUpdate, AccountContactUpdate, ContactDTO, \
    PricingLevelDTO, AcccountAddressRequestDTO, ContactRequestDTO


@dataclass(kw_only=True)
class AccountDataDTO(ABBaseDTO):
    name: str = None
    emailAddress: str = None
    status: str = None
    id: str = None
    displayName: str = None
    description: str = None
    invoiceParentAccount: str = None
    type: str = None
    imageUri: str = None
    grantPortalAccess: str = None
    website: str = None
    linkedin: str = None
    twitter: str = None
    facebook: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    parentAccount: str = None
    group: str = None
    manager: str = None
    referralTracking: str = None
    salesRep: str = None
    pricingLevel: PricingLevelDTO = None

    currency: CurrencyDTO = None
    timeZone: TimeZoneDTO = None
    tax: TaxDTO = None
    accountingCode: AccountingCodeDTO = None
    communicationPreference: list[CommunicationPreferenceDTO] = None
    paymentMethods: list[PaymentMethodsDataDTO] = None
    billingPreferences: BillingPreferencesDTO = None
    customAttributes: list[CustomAttributesDTO] = None
    try:
        addresses: list[AcccountAddressDTO] = None
        contacts: list[ContactDTO] = None
    except:
        addresses: list[AcccountAddressRequestDTO] = None
        contacts: list[ContactRequestDTO] = None
    customForms: CustomFormsDTO = None
    eventUuid: str = None
    customObjects: list[CustomObjectDTO] = None
    kpis: dict = None



@dataclass(kw_only=True)
class AccountAddressDataDTO(ABBaseDTO):
    id: str = None
    addresses: list[AcccountAddressDTO] = None


@dataclass(kw_only=True)
class AccountNoteDataDTO(ABBaseDTO):
    notes: list[NoteDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class AccountNoteResponseDataDTO(ABBaseDTO):
    notes: NoteDataDTO = None



@dataclass(kw_only=True)
class AccountAddressUuidDataDTO(ABBaseDTO):
    id: str = None
    addresses: AcccountAddressDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidDataDTO(ABBaseDTO):
    note: NoteDataDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidFileDataDTO(ABBaseDTO):
    note: NoteFileDataDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidFileUuidDataDTO(ABBaseDTO):
    note: NoteFileUuidDataDTO = None


@dataclass(kw_only=True)
class AccountCreateDTO(ABBaseDTO):
    account: AccountDataDTO


@dataclass(kw_only=True)
class AccountUpdateInformationDTO(ABBaseDTO):
    account: AccountDataDTO


@dataclass(kw_only=True)
class AccountDetailsDTO(ABBaseDTO):
    account: AccountDataDTO = None


@dataclass(kw_only=True)
class AccountImageDetailsDTO(ABBaseDTO):
    account: AccountImageDataDTO = None


@dataclass(kw_only=True)
class AccountAddressDetailsDTO(ABBaseDTO):
    account: AccountAddressDataDTO = None


@dataclass(kw_only=True)
class AccountNoteDetailsDTO(ABBaseDTO):
    account: AccountNoteDataDTO = None


@dataclass(kw_only=True)
class AccountNoteResponseDetailsDTO(ABBaseDTO):
    account: AccountNoteResponseDataDTO = None


@dataclass(kw_only=True)
class AccountAddressUuidDetailsDTO(ABBaseDTO):
    account: AccountAddressUuidDataDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidDetailsDTO(ABBaseDTO):
    account: AccountNoteUuidDataDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidFileDetailsDTO(ABBaseDTO):
    account: AccountNoteUuidFileDataDTO = None


@dataclass(kw_only=True)
class AccountNoteUuidFileUuidDetailsDTO(ABBaseDTO):
    account: AccountNoteUuidFileUuidDataDTO = None


@dataclass(kw_only=True)
class AccountReactiveResponseDTO(ABBaseDTO):
    eventUuid: str = None

@dataclass(kw_only=True)
class AccountCancelResponseDTO(ABBaseDTO):
    eventUuid: str = None


@dataclass(kw_only=True)
class AccountListDTO(ABBaseDTO):
    accounts: list[AccountDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class PaymentMethodsAddDTO(ABBaseDTO):
    account: PaymentMethodsDTO = None

    def method(self, payment_method: PaymentMethodsDataDTO):
        self.account = PaymentMethodsDTO(paymentMethod=payment_method)
        return self


@dataclass(kw_only=True)
class PaymentMethodsDetailsDTO(ABBaseDTO):
    account: PaymentMethodsDTO = None


@dataclass(kw_only=True)
class PaymentCardMethodsAddDTO(ABBaseDTO):
    account: PaymentCardMethodsDTO = None

    def method(self, payment_method: PaymentCardMethodsDataDTO):
        self.account = PaymentCardMethodsDTO(paymentMethod=payment_method)
        return self


@dataclass(kw_only=True)
class PaymentMethodsListDTO(ABBaseDTO):
    account: PaymentMethodListDTO = None


@dataclass(kw_only=True)
class AccountCancelDataDTO(ABBaseDTO):
    effectiveDate: str


@dataclass(kw_only=True)
class AccountCancelDTO(ABBaseDTO):
    account: AccountCancelDataDTO


@dataclass(kw_only=True)
class AccountReactivateDataDTO(ABBaseDTO):
    effectiveDate: str = None


@dataclass(kw_only=True)
class AccountReactivateDTO(ABBaseDTO):
    account: AccountReactivateDataDTO


@dataclass(kw_only=True)
class AccountContactsDTO(ABBaseDTO):
    account: AccountContacts = None


@dataclass(kw_only=True)
class AccountContactsTypeDTO(ABBaseDTO):
    account: AccountContactsType = None


@dataclass(kw_only=True)
class AccountContactUpdateDTO(ABBaseDTO):
    account: AccountContactUpdate = None


@dataclass(kw_only=True)
class AccountContactsUpdateDTO(ABBaseDTO):
    account: AccountContactsUpdate = None


@dataclass(kw_only=True)
class AccountAddressesAdd(ABBaseDTO):
    addresses: list[AddressDTO] = None


@dataclass(kw_only=True)
class AccountAddressesAddDTO(ABBaseDTO):
    account: AccountAddressesAdd = None


@dataclass(kw_only=True)
class AccountBillingPreferencesRequestDTO(ABBaseDTO):
    account: AccountBillingPreferencesDetailsDTO = None


@dataclass(kw_only=True)
class AccountBillingPreferencesResponseDTO(ABBaseDTO):
    account: AccountBillingPreferencesResponseDetailsDTO = None


@dataclass(kw_only=True)
class AccountAddressRequestDTO(ABBaseDTO):
    account: AccountAddressRequestDetailsDTO = None


@dataclass(kw_only=True)
class AccountAddressResponseDTO(ABBaseDTO):
    account: AccountAddressResponseDetailsDTO = None


@dataclass(kw_only=True)
class EmailSentAttachmentDTO(ABBaseDTO):
    name: str = None
    path: str = None
    size: int = None
    type: str = None


@dataclass(kw_only=True)
class EmailSentDetailsDTO(ABBaseDTO):
    uuid: str = None
    messageType: str = None
    status: str = None
    fromEmail: str = None
    to: str = None
    cc: list = None
    bcc: list = None
    subject: str = None
    body: str = None
    additionalTemplate: str = None
    createdAt: str = None
    lastUpdatedAt: str = None
    attachments: list = None

    _custom_field_mapping = {
        "from": "fromEmail"
    }


@dataclass(kw_only=True)
class AccountCommunicationDetailsDTO(ABBaseDTO):
    email: EmailSentDetailsDTO = None

    #Used only in Request as Parent
    # _custom_field_mapping = {
    #     "from": "fromEmail"
    # }


@dataclass(kw_only=True)
class EmailSentAccountDTO(ABBaseDTO):
    communication: AccountCommunicationDetailsDTO = None


@dataclass(kw_only=True)
class EmailSentResponseDTO(ABBaseDTO):
    account: EmailSentAccountDTO = None


@dataclass(kw_only=True)
class AccountCommunicationResponseListDTO(ABBaseDTO):
    emails: EmailSentDetailsDTO = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class EmailGetAccountListDTO(ABBaseDTO):
    communication: AccountCommunicationResponseListDTO = None


@dataclass(kw_only=True)
class EmailGetListResponseDTO(ABBaseDTO):
    account: EmailGetAccountListDTO = None


@dataclass(kw_only=True)
class AccountCommunicationResponseDetailsDTO(ABBaseDTO):
    email: EmailSentDetailsDTO = None


@dataclass(kw_only=True)
class EmailGetAccountDetailsDTO(ABBaseDTO):
    communication: AccountCommunicationResponseDetailsDTO = None


@dataclass(kw_only=True)
class EmailGetDetailsResponseDTO(ABBaseDTO):
    account: EmailGetAccountDetailsDTO = None



from dataclasses import dataclass

from exsited.exsited.account.dto.account_nested_dto import CommunicationPreferenceDTO, PaymentMethodsDataDTO
from exsited.exsited.common.dto.common_dto import CustomFormsDTO, CurrencyDTO, TimeZoneDTO, TaxDTO, PaginationDTO, \
    AddressDTO, ShippingProfileDTO
from exsited.exsited.order.dto.order_nested_dto import OrderLineDTO, OrderItemPriceSnapshotDTO, OrderPropertiesDTO, \
    ContractPropertiesDTO, UpgradeDowngradePreviewDTO, KpisDTO, DiscountProfileDTO, ContractAdjustmentPreviewDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class KPIStatsDTO(ABBaseDTO):
    startDate: str = None
    estimatedTotal: float = None
    totalRevenue: float = None
    monthlyRecurringRevenue: float = None
    totalCollected: float = None
    totalOutstanding: float = None
    totalDue: float = None
    lastInvoiceIssueDate: str = None
    lastInvoiceTotal: float = None
    totalInvoice: int = None
    nextInvoiceIssueDate: str = None
    lastReactivatedOn: str = None
    lastCancelledOn: str = None
    lastChangedOn: str = None
    lastDeletedOn: str = None


@dataclass(kw_only=True)
class CustomAttributesDataDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class OrderDataDTO(ABBaseDTO):
    accountId: str = None
    status: str = None
    id: str = None
    uuid: str = None
    version: str = None
    preOrder: str = None
    quoteOrder: str = None
    name: str = None
    displayName: str = None
    description: str = None
    manager: str = None
    referralAccount: str = None
    shippingCost: str = None
    origin: str = None
    invoiceNote: str = None
    billingStartDate: str = None
    orderStartDate: str = None
    nextBillingFromDate: str = None
    nextBillingFromDateUtc: str = None
    trialRequiresPaymentMethod: str = None
    priceTaxInclusive: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    allowContract: str = None

    lines: list[OrderLineDTO] = None
    customForms: CustomFormsDTO = None
    try:
        currency: CurrencyDTO = None
    except:
        currency: str = None
    timeZone: TimeZoneDTO = None
    properties: OrderPropertiesDTO = None

    contractProperties: ContractPropertiesDTO = None
    billingAddress: AddressDTO = None
    shippingAddress: AddressDTO = None
    shippingProfile: ShippingProfileDTO = None
    defaultWarehouse: str = None
    customObjects: list = None
    isTaxExemptWhenSold: str = None
    kpis: KpisDTO = None
    lineItems: list = None
    effectiveDate: str = None
    charge: OrderLineDTO = None
    communicationPreference: list[CommunicationPreferenceDTO] = None
    line: OrderLineDTO = None
    customAttributes: list[CustomAttributesDataDTO] = None
    customerPurchaseOrderId: str = None
    discountProfile: DiscountProfileDTO = None

    accountName: str = None
    currencyId: str = None
    total: str = None
    subtotal: str = None
    tax: str = None


    def add_line(self, item_id: str, quantity: str, price: str = None):
        line = OrderLineDTO(itemId=item_id, itemOrderQuantity=quantity)
        if price:
            line.itemPriceSnapshot = OrderItemPriceSnapshotDTO().add_rule(price=price)
        if not self.lines:
            self.lines = []
        self.lines.append(line)
        return self


@dataclass(kw_only=True)
class OrderCreateDTO(ABBaseDTO):
    order: OrderDataDTO


@dataclass(kw_only=True)
class OrderDowngradeDetailsDTO(ABBaseDTO):
    order: OrderDataDTO = None
    eventUuid: str = None


@dataclass(kw_only=True)
class OrderDetailsDTO(ABBaseDTO):
    order: OrderDataDTO = None
    eventUuid: str = None


@dataclass(kw_only=True)
class OrderListDTO(ABBaseDTO):
    orders: list[OrderDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class OrderCancelResponseDTO(ABBaseDTO):
    eventUuid: str = None
    order: OrderDataDTO = None


@dataclass(kw_only=True)
class AccountOrdersResponseDTO(ABBaseDTO):
    account: OrderListDTO = None


@dataclass(kw_only=True)
class OrderUpgradeDowngradeDTO(ABBaseDTO):
    preview: UpgradeDowngradePreviewDTO = None


@dataclass(kw_only=True)
class OrderUpgradeDTO(ABBaseDTO):
    effectiveDate: str = None
    effectiveImmediately: str = None
    redemptionCode: str = None
    discountPercentage: str = None
    lines: list[OrderLineDTO] = None


@dataclass(kw_only=True)
class OrderUpgradePreviewDTO(ABBaseDTO):
    effectiveDate: str = None
    effectiveImmediately: str = None
    redemptionCode: str = None
    discountPercentage: str = None
    lines: list[OrderLineDTO] = None
    properties: OrderPropertiesDTO = None


@dataclass(kw_only=True)
class OrderDowngradeDTO(ABBaseDTO):
    effectiveDate: str = None
    effectiveImmediately: str = None
    redemptionCode: str = None
    discountPercentage: str = None
    lines: list[OrderLineDTO] = None
    properties: OrderPropertiesDTO = None


@dataclass(kw_only=True)
class OrderDowngradePreviewDTO(ABBaseDTO):
    effectiveDate: str = None
    effectiveImmediately: str = None
    redemptionCode: str = None
    discountPercentage: str = None
    lines: list[OrderLineDTO] = None
    properties: OrderPropertiesDTO = None


@dataclass(kw_only=True)
class ContractAdjustmentPreviewRequestDTO(ABBaseDTO):
    effectiveDate: str = None
    effectiveImmediately: str = None
    redemptionCode: str = None
    discountPercentage: str = None
    lines: list[OrderLineDTO] = None
    properties: OrderPropertiesDTO = None


@dataclass(kw_only=True)
class ContractAdjustmentPreviewResponseDTO(ABBaseDTO):
    preview: ContractAdjustmentPreviewDTO = None


@dataclass(kw_only=True)
class OrderPaymentMethodDataDTO(ABBaseDTO):
    paymentMethods: list[PaymentMethodsDataDTO] = None

@dataclass(kw_only=True)
class OrderPaymentMethodsResponseDTO(ABBaseDTO):
    order: OrderPaymentMethodDataDTO = None




from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import CustomAttributesDTO, CustomObjectDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO

@dataclass(kw_only=True)
class PaymentAppliedDTO(ABBaseDTO):
    processor: str = None
    amount: float = None
    reference: str = None
    method: str = None

@dataclass(kw_only=True)
class CardPaymentAppliedDTO(PaymentAppliedDTO):
    cardType: str = None
    token: str = None
    cardNumber: str = None
    expiryMonth: str = None
    expiryYear: str = None
    additionalFields: dict = None

@dataclass(kw_only=True)
class CreditAppliedDTO(ABBaseDTO):
    id: str = None
    amount: str = None
    code: str = None


@dataclass(kw_only=True)
class giftCertificateAppliedDTO(ABBaseDTO):
    code: str = None
    amount: str = None


@dataclass(kw_only=True)
class InvoiceDTO(ABBaseDTO):
    applied: float = None
    code: str = None
    dueDate: str = None
    issueDate: str = None
    outstanding: float = None
    total: float = None

@dataclass(kw_only=True)
class PaymentDataDTO(ABBaseDTO):
    id: str = None
    date: str = None
    status: str = None
    reconcileStatus: str = None
    totalApplied: float = None
    paymentApplied: list[PaymentAppliedDTO] = None
    note: str = None
    creditApplied: list[CreditAppliedDTO] = None
    invoices: list[InvoiceDTO] = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    saleOrderId: str = None
    customAttributes: list[CustomAttributesDTO] = None
    customObjects: list[CustomObjectDTO] = None
    giftCertificateApplied: list[giftCertificateAppliedDTO] = None


@dataclass(kw_only=True)
class CardPaymentDataDTO(PaymentDataDTO):
    paymentApplied: list[CardPaymentAppliedDTO] = None

@dataclass(kw_only=True)
class PaymentDetailsDTO(ABBaseDTO):
    payment: PaymentDataDTO = None

@dataclass(kw_only=True)
class PaymentCreateDTO(ABBaseDTO):
    payment: PaymentDataDTO = None

@dataclass(kw_only=True)
class CardPaymentCreateDTO(ABBaseDTO):
    payment: CardPaymentDataDTO = None

# New DTO for the given payload
@dataclass(kw_only=True)
class CardDirectDebitPaymentAppliedDTO(PaymentAppliedDTO):
    method: str = None

@dataclass(kw_only=True)
class CardDirectDebitPaymentDataDTO(ABBaseDTO):
    date: str = None
    paymentApplied: list[CardDirectDebitPaymentAppliedDTO] = None

@dataclass(kw_only=True)
class CardDirectDebitPaymentCreateDTO(ABBaseDTO):
    payment: CardDirectDebitPaymentDataDTO = None


@dataclass(kw_only=True)
class PaginationDTO(ABBaseDTO):
    records: int = None
    limit: int = None
    offset: int = None
    previousPage: str = None
    nextPage: str = None

@dataclass(kw_only=True)
class PaymentListDTO(ABBaseDTO):
    payments: list[PaymentDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class PaymentInvoiceResponseDTO(ABBaseDTO):
    invoice: PaymentListDTO = None


@dataclass(kw_only=True)
class PaymentAccountResponseDTO(ABBaseDTO):
    account: PaymentListDTO = None


@dataclass(kw_only=True)
class PaymentOrderResponseDTO(ABBaseDTO):
    order: PaymentListDTO = None



@dataclass(kw_only=True)
class CustomAttributeDTO:
    name: str
    value: str

@dataclass(kw_only=True)
class InvoiceMultipleDTO:
    id: str
    amount: str

@dataclass(kw_only=True)
class CreditAppliedDTO:
    id: str
    amount: str

@dataclass(kw_only=True)
class GiftCertificateDTO:
    code: str
    applied: str

@dataclass(kw_only=True)
class AdditionalFieldsDTO:
    hostIp: str

@dataclass(kw_only=True)
class PaymentAppliedMultipleDTO:
    processor: str
    amount: str
    cardType: str
    token: str
    cardNumber: str
    expiryMonth: str
    expiryYear: str
    additionalFields: list[AdditionalFieldsDTO] = None

@dataclass(kw_only=True)
class PaymentMultipleDataDTO:
    id: str
    invoices: list[InvoiceMultipleDTO]
    date: str
    paymentApplied: PaymentAppliedMultipleDTO
    creditApplied: list[CreditAppliedDTO]
    giftCertificates: list[GiftCertificateDTO]
    customAttributes: list[CustomAttributeDTO]

@dataclass(kw_only=True)
class PaymentMultipleRequestDTO(ABBaseDTO):
    payment: PaymentMultipleDataDTO = None


from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import CustomAttributesDTO, CustomObjectDTO, PaginationDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class PurchasePaymentAppliedDTO(ABBaseDTO):
    id: str = None
    amount: str = None
    method: str = None
    processor: str = None
    reference: str = None


@dataclass(kw_only=True)
class PurchaseCreditAppliedDTO(ABBaseDTO):
    id: str = None
    amount: float = None


@dataclass(kw_only=True)
class PurchaseInvoiceDTO(ABBaseDTO):
    applied: float = None
    id: str = None
    dueDate: str = None
    issueDate: str = None
    outstanding: float = None
    total: float = None
    amount: str = None


@dataclass(kw_only=True)
class PurchasePaymentDTO(ABBaseDTO):
    status: str = None
    id: str = None
    purchasePaymentDate: str = None
    totalApplied: str = None
    purchaseOrderId: str = None
    purchasePaymentNote: str = None
    purchasePaymentApplied: list[PurchasePaymentAppliedDTO] = None
    purchaseCreditApplied: list[PurchaseCreditAppliedDTO] = None
    purchaseInvoices: list[PurchaseInvoiceDTO] = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    customAttributes: list[CustomAttributesDTO] = None
    customObjects: list[CustomObjectDTO] = None
    date: str = None
    note: str = None
    effectiveDate: str = None


@dataclass(kw_only=True)
class PurchasePaymentUpdateDataDTO(ABBaseDTO):
    date: str = None
    note: str = None
    customAttributes: list[CustomAttributesDTO] = None


@dataclass(kw_only=True)
class PurchasePaymentUpdateRequestDTO(ABBaseDTO):
    purchasePayment: PurchasePaymentUpdateDataDTO = None


@dataclass(kw_only=True)
class PurchasePaymentsListDTO(ABBaseDTO):
    purchasePayments: list[PurchasePaymentDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class PurchasePaymentsDetailsDTO(ABBaseDTO):
    purchasePayments: PurchasePaymentDTO = None


@dataclass(kw_only=True)
class PurchasePaymentsUpdateResponseDTO(ABBaseDTO):
    purchasePayment: PurchasePaymentDTO = None


@dataclass(kw_only=True)
class PurchasePaymentRequestDTO(ABBaseDTO):
    purchasePayment: PurchasePaymentDTO = None

@dataclass(kw_only=True)
class PurchasePaymentResponseDTO(ABBaseDTO):
    purchasePayment: PurchasePaymentDTO = None



class InvoiceApiUrl:
    INVOICES = "/api/v3/invoices"
    INVOICE_CREATE = "/api/v2/orders/{id}/invoices"
    EACH_INVOICE = "/api/v2/orders/{id}/invoices"
    EACH_INVOICE_V3 = "/api/v3/orders/{id}/invoices"
    INVOICE_INFORMATION = '/api/v2/invoices/{id}/information'
    INVOICE_ACCOUNT = '/api/v3/accounts/{id}/invoices'
    INVOICE_DELETE = '/api/v2/invoices/{id}'
    INVOICE_UPDATE_AMEND = '/api/v2/invoices/{id}/amend'



from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import CustomAttributesDTO, PaginationDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO
@dataclass(kw_only=True)
class UsageDataDTO(ABBaseDTO):
    uuid: str = None
    version: str = None
    chargeItemUuid: str = None
    chargeItemName: str = None
    chargingPeriod: str = None
    quantity: str = None
    uom: str = None
    startTime: str = None
    endTime: str = None
    type: str = None
    chargeStatus: str = None
    source: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    customAttributes: list[CustomAttributesDTO] = None
    usageReference: str = None


@dataclass(kw_only=True)
class UsageCreateDTO(ABBaseDTO):
    usage: UsageDataDTO = None


@dataclass(kw_only=True)
class MultipleUsageCreateDTO(ABBaseDTO):
    usages: list[UsageDataDTO] = None
    pagination: PaginationDTO = None


@dataclass(kw_only=True)
class MultipleUsageResponseDTO(ABBaseDTO):
    success: list[UsageDataDTO] = None
    failed: list[UsageDataDTO] = None


@dataclass(kw_only=True)
class UsageListDTO(ABBaseDTO):
    usages: list[UsageDataDTO] = None
    pagination: PaginationDTO = None

@dataclass(kw_only=True)
class UsageModifyDataDTO(ABBaseDTO):
    usage: UsageDataDTO = None

@dataclass(kw_only=True)
class UsageUpdateDataDTO(ABBaseDTO):
    usage: UsageDataDTO = None




import dataclasses
from enum import Enum
import re
from types import GenericAlias


class PropsModType(Enum):
    CAMEL_TO_SNAKE = "CAMEL_TO_SNAKE"
    SNAKE_TO_CAMEL = "SNAKE_TO_CAMEL"


class DTOBase(object):
    _exclude_props_mod: dict = {}
    _props_mod_type: PropsModType = None
    _camel_to_snake_regex = re.compile(r"(?<!^)(?=[A-Z])")
    _include_null: bool = False

    def convert_snake_to_camel_case(self, text: str) -> str:
        camel_cased = "".join(x.capitalize() for x in text.lower().split("_"))
        if camel_cased:
            return camel_cased[0].lower() + camel_cased[1:]
        else:
            return camel_cased

    def convert_camel_to_snake_case(self, text: str) -> str:
        return self._camel_to_snake_regex.sub("_", text).lower()

    def _get_property_modifier(self):
        if not self._props_mod_type:
            return None
        elif self._props_mod_type == PropsModType.CAMEL_TO_SNAKE:
            return self.convert_camel_to_snake_case
        elif self._props_mod_type == PropsModType.SNAKE_TO_CAMEL:
            return self.convert_snake_to_camel_case
        return None

    def _load_data_class(cls, data: dict):
        if not dataclasses.is_dataclass(cls):
            raise ValueError(f"{cls.__name__} must be a dataclass")
        if not isinstance(data, dict):
            raise ValueError(f"Unable to load dictionary from {cls.__name__}")
        cls.set_deserialize_conf(cls)
        kwargs = cls._get_load_kwargs(cls, data=data)
        return cls(**kwargs)

    def _get_collection_args(self, object_class, index):
        if hasattr(object_class, "__args__") and len(getattr(object_class, "__args__")) > index:
            return getattr(object_class, "__args__")[index]
        return None

    def _load_generic_alias_data(cls, field_type, values):
        generic_class_name = cls._get_collection_args(cls, field_type, 0)
        object_list = []
        for value in values:
            object_list.append(cls._load_data_class(generic_class_name, data=value))
        return object_list

    def _get_load_kwargs(cls, data: dict):
        property_modifier = cls._get_property_modifier(cls)
        field_name_type = {field.name: field.type for field in dataclasses.fields(cls)}
        kwargs = {}
        for key, value in data.items():
            if property_modifier and (not cls._exclude_props_mod or key not in cls._exclude_props_mod):
                key = property_modifier(cls, text=key)
            if key in field_name_type:
                field_type = field_name_type[key]
                if isinstance(value, list) and isinstance(field_type, GenericAlias):
                    kwargs[key] = cls._load_generic_alias_data(cls, field_type=field_type, values=value)
                elif isinstance(value, dict) and issubclass(field_type, DTOBase):
                    kwargs[key] = cls._load_data_class(field_type, data=value)
                else:
                    kwargs[key] = value
        return kwargs

    @classmethod
    def load_dict(cls, data: dict):
        return cls._load_data_class(cls, data=data)

    def _dict_factory(self, fields):
        dict_field = {}
        property_modifier = self._get_property_modifier()
        for (key, value) in fields:
            if property_modifier and (not self._exclude_props_mod or key not in self._exclude_props_mod):
                key = property_modifier(text=key)
            if value is not None or self._include_null:
                dict_field[key] = value
        return dict_field

    def to_dict(self, include_null=False):
        self._include_null = include_null
        self.set_serialize_conf()
        return dataclasses.asdict(self, dict_factory=self._dict_factory)

    def set_serialize_conf(self):
        pass

    def set_deserialize_conf(self):
        pass



from exsited.exsited.common.common_enum import SortDirection
from exsited.common.sdk_util import SDKUtil
from exsited.exsited.credit_note.credit_note_api_url import CreditNoteApiUrl
from exsited.exsited.credit_note.dto.credit_note_dto import CreditNoteDTO, CreditNoteDetailsDTO, \
    CreditNoteApplicationListDTO, CreditNoteApplicationDetailDTO, InvoiceCreditNoteApplicationListDTO
from exsited.http.ab_rest_processor import ABRestProcessor


class CreditNote(ABRestProcessor):
    def details(self, id: str) -> CreditNoteDetailsDTO:
        response = self.get(url=CreditNoteApiUrl.DETAILS.format(id=id), response_obj=CreditNoteDetailsDTO())
        return response

    def credit_note_application_list(self) -> CreditNoteApplicationListDTO:
        response = self.get(url=CreditNoteApiUrl.APPLICATION_LIST, response_obj=CreditNoteApplicationListDTO())
        return response

    def credit_note_application_details(self, uuid: str) -> CreditNoteApplicationDetailDTO:
        response = self.get(url=CreditNoteApiUrl.APPLICATION_DETAILS.format(uuid=uuid), response_obj=CreditNoteApplicationDetailDTO())
        return response

    def invoice_credit_note_application_list(self, id: str) -> InvoiceCreditNoteApplicationListDTO:
        response = self.get(url=CreditNoteApiUrl.INVOICE_CN_APPLICATION_LIST.format(id=id), response_obj=InvoiceCreditNoteApplicationListDTO())
        return response



class ReturnMerchandiseAuthorisationsApiUrl:
    LIST = '/api/v3/return-merchandise-authorisations'
    RECEIVE_LIST = '/api/v2/invoices/{id}/return-merchandise-authorisations/{rma_id}/receive'
    INVOICE_RMA_LIST = "/api/v2/invoices/{id}/return-merchandise-authorisations"
    INVOICE_RMA_DETAILS = "/api/v2/invoices/{id}/return-merchandise-authorisations/{rma_id}"
    INVOICE_RMA_CREATE = "/api/v3/invoices/{id}/return-merchandise-authorisations"
    INVOICE_RECEIVE_RMA_CREATE = "/api/v2/invoices/{id}/return-merchandise-authorisations/{rma_id}/receive"
    RECEIVE_DETAILS = "/api/v2/invoices/{id}/return-merchandise-authorisations/{rma_id}/receive/{rec_id}"



import requests
from exsited.http.http_const import HTTPConst


class HTTPResponse:
    status: str
    httpCode: int
    data: any
    contentType: str

    def __init__(self):
        self.status = None
        self.httpCode = None
        self.data = None
        self.contentType = None

    def process_response(self, response) -> "HTTPResponse":
        self.httpCode = response.status_code
        if 200 <= self.httpCode < 300:
            self.status = HTTPConst.SUCCESS
        else:
            self.status = HTTPConst.ERROR
        self.set_data(response)
        return self

    def set_data(self, response):
        response.encoding = HTTPConst.UTF8_ENCODING
        self.contentType = response.headers["Content-Type"]
        if self.contentType and self.contentType.lower().startswith(HTTPConst.APPLICATION_JSON):
            if response.text and response.text != "":
                self.data = response.json()
        else:
            self.data = response.text

    @staticmethod
    def get_response(response) -> "HTTPResponse":
        http_response = HTTPResponse()
        http_response.process_response(response)
        return http_response


class HTTPRequester:
    headers: dict = {}
    baseUrl: str

    def _get_url(self, url):
        return self.baseUrl + url

    def set_base(self, url) -> 'HTTPRequester':
        self.baseUrl = url
        return self

    def get(self, url: str, params: dict = None) -> HTTPResponse:
        url = self._get_url(url)
        response = requests.get(url, headers=self.headers, params=params)
        return HTTPResponse.get_response(response)

    def post(self, url: str, json_dict: dict = None, data: dict = None, file: dict = None) -> HTTPResponse:
        url = self._get_url(url)
        response = requests.post(url, headers=self.headers, json=json_dict, data=data, files=file)
        return HTTPResponse.get_response(response)

    def put(self, url: str, json_dict: dict = None, data: dict = None, file: dict = None) -> HTTPResponse:
        url = self._get_url(url)
        response = requests.put(url, headers=self.headers, json=json_dict, data=data, files=file)
        return HTTPResponse.get_response(response)

    def patch(self, url: str, json_dict: dict = None, data: dict = None, file: dict = None) -> HTTPResponse:
        url = self._get_url(url)
        response = requests.patch(url, headers=self.headers, json=json_dict, data=data, files=file)
        return HTTPResponse.get_response(response)

    def delete(self, url: str, params: dict = None) -> HTTPResponse:
        url = self._get_url(url)
        response = requests.delete(url, headers=self.headers, params=params)
        return HTTPResponse.get_response(response)

    def add_header(self, key: str, value) -> 'HTTPRequester':
        self.headers[key] = value
        return self

    def add_bearer_token(self, token) -> 'HTTPRequester':
        self.add_header("Authorization", f"Bearer {token}")
        return self

    def add_content_type(self, content_type) -> 'HTTPRequester':
        self.add_header("Content-Type", str(content_type))
        return self



from exsited.exsited.refund.dto.refund_dto import RefundDetailsDTO, AccountRefundListDTO
from exsited.exsited.refund.refund_api_url import (RefundApiUrl)
from exsited.http.ab_rest_processor import ABRestProcessor


class Refund(ABRestProcessor):

    def details(self, id: str) -> RefundDetailsDTO:
        response = self.get(url=RefundApiUrl.DETAILS.format(id=id), response_obj=RefundDetailsDTO())
        return response

    def account_refund_list(self, id: str) -> AccountRefundListDTO:
        response = self.get(url=RefundApiUrl.ACCOUNT_REFUND_LIST.format(id=id), response_obj=AccountRefundListDTO())
        return response

    def create(self, cn_id: str, request_data: RefundDetailsDTO) -> RefundDetailsDTO:
        response = self.post(url=RefundApiUrl.CREATE.format(cn_id=cn_id), request_obj=request_data, response_obj=RefundDetailsDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=RefundApiUrl.REFUND_DELETE.format(id=id))
        return response




from dataclasses import dataclass
from exsited.sdlize.ab_base_dto import ABBaseDTO
from typing import Union

@dataclass(kw_only=True)
class PaymentProcessorDetailsDTO(ABBaseDTO):
    uuid: str = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class SpecifiedOrdersDTO(ABBaseDTO):
    orderId: str = None


@dataclass(kw_only=True)
class PaymentMethodsDataDTO(ABBaseDTO):
    processorType: str = None
    paymentProcessor: str = None
    status: str = None
    default: str = None
    reference: str = None
    paymentCount: str = None
    lastUsedOn: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    specifiedOrders: list[Union[str, SpecifiedOrdersDTO]] = None
    useForSpecifiedOrders: str = None
    processor: PaymentProcessorDetailsDTO = None

    cardType: str = None
    token: str = None
    cardNumber: str = None
    expiryMonth: str = None
    expiryYear: str = None
    cardCvv: str = None
    nameOnCard: str = None
    lastUsedResult: str = None
    errorCountSinceLastSuccess: str = None


@dataclass(kw_only=True)
class PaymentMethodsDTO(ABBaseDTO):
    id: str = None
    paymentMethod: PaymentMethodsDataDTO = None


@dataclass(kw_only=True)
class PaymentMethodListDTO(ABBaseDTO):
    id: str = None
    paymentMethods: list[PaymentMethodsDataDTO] = None


@dataclass(kw_only=True)
class PaymentCardMethodsDataDTO(PaymentMethodsDataDTO):
    cardType: str = None
    token: str = None
    cardNumber: str = None
    expiryMonth: str = None
    expiryYear: str = None


@dataclass(kw_only=True)
class AdditionalFieldsDTO(ABBaseDTO):
    hostIp: str = None


@dataclass(kw_only=True)
class PaymentCardDirectDebitDataDTO(PaymentMethodsDataDTO):
    processorType: str = None
    default: str = None
    paymentProcessor: str = None
    accountNumber: str = None
    routingNumber: str = None
    accountName: str = None
    reference: str = None
    additionalFields: AdditionalFieldsDTO = None


@dataclass(kw_only=True)
class PaymentCardMethodsDTO(ABBaseDTO):
    paymentMethod: PaymentCardMethodsDataDTO


@dataclass(kw_only=True)
class BillingPreferencesDTO(ABBaseDTO):
    communicationProfile: str = None
    invoiceMode: str = None
    invoiceTerm: str = None
    billingPeriod: str = None
    billingStartDate: str = None
    billingStartDayOfMonth: str = None
    chargingAndBillingAlignment: str = None
    paymentProcessor: str = None
    paymentMode: str = None
    paymentTerm: str = None
    paymentTermAlignment: str = None


@dataclass(kw_only=True)
class CommunicationPreferenceDTO(ABBaseDTO):
    media: str = None
    isEnabled: bool = None


@dataclass(kw_only=True)
class AccountingCodeDTO(ABBaseDTO):
    accountReceivable: str = None


@dataclass(kw_only=True)
class SalutationDTO(ABBaseDTO):
    id: int = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class DesignationDTO(ABBaseDTO):
    id: int = None
    name: str = None
    link: str = None


@dataclass(kw_only=True)
class EmailDTO(ABBaseDTO):
    address: str = None
    doNotEmail: str = None


@dataclass(kw_only=True)
class AccountImageDataDTO(ABBaseDTO):
    imageUri: str = None
    accountId: str = None


@dataclass(kw_only=True)
class PhoneDTO(ABBaseDTO):
    countryCode: str = None
    areaCode: str = None
    number: str = None
    full: str = None
    doNotEmail: str = None
    doNotCall: str = None


@dataclass(kw_only=True)
class CustomAttributeDTO(ABBaseDTO):
    name: str = None
    value: str = None


@dataclass(kw_only=True)
class ContactRequestDTO(ABBaseDTO):
    type: str = None
    typeDisplayName: str = None
    billingContact: bool = None
    shippingContact: bool = None
    salutation: SalutationDTO = None
    designation: DesignationDTO = None
    firstName: str = None
    middleName: str = None
    lastName: str = None
    email: EmailDTO = None
    addressLine_1: str = None
    addressLine_2: str = None
    addressLine_3: str = None
    addressLine_4: str = None
    addressLine_5: str = None
    postCode: str = None
    city: str = None
    state: str = None
    country: str = None
    phone: PhoneDTO = None
    try:
        fax:  PhoneDTO = None
        mobile:  PhoneDTO = None
    except:
        fax: str = None
        mobile: str = None
    receiveBillingInformation: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    customAttributes: list[CustomAttributeDTO] = None


@dataclass(kw_only=True)
class PricingLevelDTO(ABBaseDTO):
    name: str = None
    uuid: str = None


@dataclass(kw_only=True)
class ContactDTO(ABBaseDTO):
    type: str = None
    typeDisplayName: str = None
    billingContact: bool = None
    shippingContact: bool = None
    salutation: SalutationDTO = None
    designation: DesignationDTO = None
    firstName: str = None
    middleName: str = None
    lastName: str = None
    email: EmailDTO = None
    addressLine1: str = None
    addressLine2: str = None
    addressLine3: str = None
    addressLine4: str = None
    addressLine5: str = None
    postCode: str = None
    city: str = None
    state: str = None
    country: str = None
    phone: PhoneDTO = None
    try:
        fax:  PhoneDTO = None
        mobile:  PhoneDTO = None
    except:
        fax: str = None
        mobile: str = None
    receiveBillingInformation: str = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None
    uuid: str = None
    version: str = None
    customAttributes: list[CustomAttributeDTO] = None


@dataclass(kw_only=True)
class AccountContacts(ABBaseDTO):
    id: str = None  # Default value set to None
    contacts: list[ContactDTO] = None


@dataclass(kw_only=True)
class AccountContactsType(ABBaseDTO):
    id: str = None  # Default value set to None
    contacts: ContactDTO = None


@dataclass(kw_only=True)
class AccountContactUpdate(ABBaseDTO):
    contact: ContactDTO = None


@dataclass(kw_only=True)
class AccountContactsUpdate(ABBaseDTO):
    contacts: ContactDTO = None
    id: str = None

@dataclass(kw_only=True)
class AcccountAddressDTO(ABBaseDTO):
    addressLine1: str = None
    addressLine2: str = None
    addressLine3: str = None
    addressLine4: str = None
    addressLine5: str = None
    postCode: str = None
    uuid: str = None
    city: str = None
    state: str = None
    country: str = None
    isDefaultBilling: bool = None
    isDefaultShipping: bool = None


@dataclass(kw_only=True)
class AcccountAddressRequestDTO(ABBaseDTO):
    address_line_1: str = None
    address_line_2: str = None
    address_line_3: str = None
    address_line_4: str = None
    address_line_5: str = None
    postCode: str = None
    uuid: str = None
    city: str = None
    state: str = None
    country: str = None
    isDefaultBilling: bool = None
    isDefaultShipping: bool = None


@dataclass(kw_only=True)
class FileDTO(ABBaseDTO):
    uuid: str = None
    name: str = None
    version: str = None

@dataclass(kw_only=True)
class NoteDataDTO(ABBaseDTO):
    uuid: str = None
    version: str = None
    content: str = None
    files: list[FileDTO] = None
    createdBy: str = None
    createdOn: str = None
    lastUpdatedBy: str = None
    lastUpdatedOn: str = None


@dataclass(kw_only=True)
class NoteFileDataDTO(ABBaseDTO):
    files: list[FileDTO] = None


@dataclass(kw_only=True)
class NoteFileUuidDataDTO(ABBaseDTO):
    file: FileDTO = None

##############
@dataclass(kw_only=True)
class AccountBillingPreferencesDetailsDTO(ABBaseDTO):
    billingPreferences: BillingPreferencesDTO = None

@dataclass(kw_only=True)
class AccountBillingPreferencesResponseDetailsDTO(ABBaseDTO):
    id: str = None
    billingPreferences: BillingPreferencesDTO = None


@dataclass(kw_only=True)
class AccountAddressRequestDetailsDTO(ABBaseDTO):
    address: AcccountAddressRequestDTO = None


@dataclass(kw_only=True)
class AccountAddressResponseDetailsDTO(ABBaseDTO):
    id: str = None
    addresses: AcccountAddressDTO = None



from exsited.common.ab_exception import ABException
from exsited.exsited.exsited_sdk import ExsitedSDK
from tests.common.common_data import CommonData
from tests.order_test import test_order_details


class UsageService:
    CALL_ITEM_NAME = "Example Billable Calls"
    MESSAGE_ITEM_NAME = "Example Billable Messages"

    def process_usage_data(self, transformed_usage_data):
        results = []

        for data in transformed_usage_data:
            result = {}

            if "call_id" in data:
                result['itemName'] = self.CALL_ITEM_NAME
                result['callStart'] = data.get('call_start')
                result['callDuration_sec'] = data.get('call_duration_sec')
                result['callDestination'] = data.get('call_destination')
            else:
                result['itemName'] = self.MESSAGE_ITEM_NAME
                result['billingPeriod'] = data.get('billing_period')
                result['messagesSent'] = data.get('messages_sent')
                result['includedMessages'] = data.get('included_messages')

            response = self.get_order_details(data.get('order_id'))
            if response and response.order:
                for line in response.order.lines:
                    if line.itemName == result['itemName']:
                        result['itemType'] = line.itemType
                        result['chargeItemUuid'] = line.chargeItemUuid

            if result.get('chargeItemUuid'):
                results.append(result)

        return results

    def get_order_details(self, id: str):
        exsited_sdk: ExsitedSDK = ExsitedSDK().init_sdk(request_token_dto=CommonData.get_request_token_dto())

        try:
            response = exsited_sdk.order.details(id=id)
            return response
        except ABException as ab:
            error_code = None
            if ab.get_errors() and "errors" in ab.raw_response:
                error_code = ab.raw_response["errors"][0].get("code", None)
            print(f"Error occurred while fetching order details: {error_code}")
            return None
